@@ -31,6 +31,7 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
@@ -38,6 +39,9 @@
 static GHashTable *files = NULL;
 
 static char *dot_dir = NULL;
+#ifndef HAVE_G_GET_USER_SPECIAL_DIR
+static char *music_dir = NULL;
+#endif
 
 const char *
 rb_file (const char *filename)
@@ -90,6 +94,158 @@ rb_dot_dir (void)
 	return dot_dir;
 }
 
+
+#ifndef HAVE_G_GET_USER_SPECIAL_DIR
+
+/* Copied from xdg-user-dir-lookup.c */
+
+static char *
+xdg_user_dir_lookup (const char *type)
+{
+  FILE *file;
+  char *home_dir, *config_home, *config_file;
+  char buffer[512];
+  char *user_dir;
+  char *p, *d;
+  int len;
+  int relative;
+  
+  home_dir = getenv ("HOME");
+
+  if (home_dir == NULL)
+    return strdup ("/tmp");
+
+  config_home = getenv ("XDG_CONFIG_HOME");
+  if (config_home == NULL || config_home[0] == 0)
+    {
+      config_file = malloc (strlen (home_dir) + strlen ("/.config/user-dirs.dirs") + 1);
+      strcpy (config_file, home_dir);
+      strcat (config_file, "/.config/user-dirs.dirs");
+    }
+  else
+    {
+      config_file = malloc (strlen (config_home) + strlen ("/user-dirs.dirs") + 1);
+      strcpy (config_file, config_home);
+      strcat (config_file, "/user-dirs.dirs");
+    }
+
+  file = fopen (config_file, "r");
+  free (config_file);
+  if (file == NULL)
+    goto error;
+
+  user_dir = NULL;
+  while (fgets (buffer, sizeof (buffer), file))
+    {
+      /* Remove newline at end */
+      len = strlen (buffer);
+      if (len > 0 && buffer[len-1] == '\n')
+	buffer[len-1] = 0;
+      
+      p = buffer;
+      while (*p == ' ' || *p == '\t')
+	p++;
+      
+      if (strncmp (p, "XDG_", 4) != 0)
+	continue;
+      p += 4;
+      if (strncmp (p, type, strlen (type)) != 0)
+	continue;
+      p += strlen (type);
+      if (strncmp (p, "_DIR", 4) != 0)
+	continue;
+      p += 4;
+
+      while (*p == ' ' || *p == '\t')
+	p++;
+
+      if (*p != '=')
+	continue;
+      p++;
+      
+      while (*p == ' ' || *p == '\t')
+	p++;
+
+      if (*p != '"')
+	continue;
+      p++;
+      
+      relative = 0;
+      if (strncmp (p, "$HOME/", 6) == 0)
+	{
+	  p += 6;
+	  relative = 1;
+	}
+      else if (*p != '/')
+	continue;
+      
+      if (relative)
+	{
+	  user_dir = malloc (strlen (home_dir) + 1 + strlen (p) + 1);
+	  strcpy (user_dir, home_dir);
+	  strcat (user_dir, "/");
+	}
+      else
+	{
+	  user_dir = malloc (strlen (p) + 1);
+	  *user_dir = 0;
+	}
+      
+      d = user_dir + strlen (user_dir);
+      while (*p && *p != '"')
+	{
+	  if ((*p == '\\') && (*(p+1) != 0))
+	    p++;
+	  *d++ = *p++;
+	}
+      *d = 0;
+    }  
+  fclose (file);
+
+  if (user_dir)
+    return user_dir;
+
+ error:
+  /* Special case desktop for historical compatibility */
+  if (strcmp (type, "DESKTOP") == 0)
+    {
+      user_dir = malloc (strlen (home_dir) + strlen ("/Desktop") + 1);
+      strcpy (user_dir, home_dir);
+      strcat (user_dir, "/Desktop");
+      return user_dir;
+    }
+  else
+    return strdup (home_dir);
+}
+
+const char *
+rb_music_dir (void)
+{
+	if (music_dir != NULL)
+		return (const char *) music_dir;
+	music_dir = xdg_user_dir_lookup ("MUSIC");
+	return (const char *) music_dir;
+}
+
+#else		/* HAVE_G_GET_USER_SPECIAL_DIR */
+
+const char *
+rb_music_dir (void)
+{
+	const char *dir;
+	dir = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
+	if (dir == NULL) {
+		dir = getenv ("HOME");
+		if (dir == NULL) {
+			dir = "/tmp";
+		}
+	}
+	rb_debug ("user music dir: %s", dir);
+	return dir;
+}
+
+#endif
+
 void
 rb_file_helpers_init (void)
 {
@@ -103,8 +259,10 @@ void
 rb_file_helpers_shutdown (void)
 {
 	g_hash_table_destroy (files);
-
 	g_free (dot_dir);
+#ifndef HAVE_G_GET_USER_SPECIAL_DIR
+	g_free (music_dir);
+#endif
 }
 
 #define MAX_LINK_LEVEL 5
@@ -610,6 +768,109 @@ rb_uri_is_local (const char *text_uri)
 	return g_str_has_prefix (text_uri, "file://");
 }
 
+gboolean
+rb_uri_is_hidden (const char *text_uri)
+{
+	return g_utf8_strrchr (text_uri, -1, GNOME_VFS_URI_PATH_CHR)[1] == '.';
+}
+
+gboolean
+rb_uri_could_be_podcast (const char *uri, gboolean *is_opml)
+{
+	const char *query_string;
+
+	if (is_opml != NULL)
+		*is_opml = FALSE;
+
+	/* Check the scheme is a possible one first */
+	if (g_str_has_prefix (uri, "http") == FALSE &&
+	    g_str_has_prefix (uri, "itpc:") == FALSE &&
+	    g_str_has_prefix (uri, "itms:") == FALSE) {
+	    	rb_debug ("'%s' can't be a Podcast or OPML file, not the right scheme", uri);
+	    	return FALSE;
+	}
+
+	/* Now, check whether the iTunes Music Store link
+	 * is a podcast */
+	if (g_str_has_prefix (uri, "itms:") != FALSE
+	    && strstr (uri, "phobos.apple.com") != NULL
+	    && strstr (uri, "viewPodcast") != NULL)
+		return TRUE;
+
+	query_string = strchr (uri, '?');
+	if (query_string == NULL) {
+		query_string = uri + strlen (uri);
+	}
+
+	/* FIXME hacks */
+	if (strstr (uri, "rss") != NULL ||
+	    strstr (uri, "atom") != NULL ||
+	    strstr (uri, "feed") != NULL) {
+	    	rb_debug ("'%s' should be Podcast file, HACK", uri);
+	    	return TRUE;
+	} else if (strstr (uri, "opml") != NULL) {
+		rb_debug ("'%s' should be an OPML file, HACK", uri);
+		if (is_opml != NULL)
+			*is_opml = TRUE;
+		return TRUE;
+	}
+
+	if (strncmp (query_string - 4, ".rss", 4) == 0 ||
+	    strncmp (query_string - 4, ".xml", 4) == 0 ||
+	    strncmp (query_string - 5, ".atom", 5) == 0 ||
+	    strncmp (uri, "itpc", 4) == 0 ||
+	    (strstr (uri, "phobos.apple.com/") != NULL && strstr (uri, "viewPodcast") != NULL) ||
+	    strstr (uri, "itunes.com/podcast") != NULL) {
+	    	rb_debug ("'%s' should be Podcast file", uri);
+	    	return TRUE;
+	} else if (strncmp (query_string - 5, ".opml", 5) == 0) {
+		rb_debug ("'%s' should be an OPML file", uri);
+		if (is_opml != NULL)
+			*is_opml = TRUE;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+char *
+rb_uri_make_hidden (const char *text_uri)
+{
+	GnomeVFSURI *uri;
+	GnomeVFSURI *parent;
+	char *shortname;
+	char *dotted;
+	char *ret;
+
+	if (rb_uri_is_hidden (text_uri))
+		return g_strdup (text_uri);
+
+	uri = gnome_vfs_uri_new (text_uri);
+	if (uri == NULL) {
+		return g_strdup (text_uri);
+	}
+
+	parent = gnome_vfs_uri_get_parent (uri);
+	if (parent == NULL) {
+		gnome_vfs_uri_unref (uri);
+		return g_strdup (text_uri);
+	}
+
+	shortname = gnome_vfs_uri_extract_short_name (uri);
+	gnome_vfs_uri_unref (uri);
+
+	dotted = g_strdup_printf (".%s", shortname);
+	g_free (shortname);
+
+	uri = gnome_vfs_uri_append_file_name (parent, dotted);
+	gnome_vfs_uri_unref (parent);
+	g_free (dotted);
+
+	ret = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+	gnome_vfs_uri_unref (uri);
+	return ret;
+}
+
 /*
  * gnome_vfs_uri_new escapes a few extra characters that
  * gnome_vfs_escape_path doesn't ('&' and '=').  If we
@@ -629,12 +890,43 @@ escape_extra_gnome_vfs_chars (char *uri)
 }
 
 typedef struct {
-	const char *uri;
-	GFunc func;
+	char *uri;
+	RBUriRecurseFunc func;
 	gpointer user_data;
 	gboolean *cancel_flag;
+	GDestroyNotify data_destroy;
 } RBUriHandleRecursivelyData;
 
+typedef struct {
+	RBUriHandleRecursivelyData data;
+
+	/* real data */
+	RBUriRecurseFunc func;
+	gpointer user_data;
+
+	GMutex *results_lock;
+	guint results_idle_id;
+	GList *uri_results;
+	GList *dir_results;
+} RBUriHandleRecursivelyAsyncData;
+
+static void
+_rb_uri_recurse_data_free (RBUriHandleRecursivelyData *data)
+{
+	g_free (data->uri);
+	if (data->data_destroy)
+		data->data_destroy (data->user_data);
+
+	g_free (data);
+}
+
+static gboolean
+_gnomevfs_info_unreadable (GnomeVFSFileInfo *info)
+{
+	return (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
+		!(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE);
+}
+
 static gboolean
 rb_uri_handle_recursively_cb (const gchar *rel_path,
 			      GnomeVFSFileInfo *info,
@@ -642,35 +934,40 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 			      RBUriHandleRecursivelyData *data,
 			      gboolean *recurse)
 {
+	char *path, *escaped_rel_path;
+	char *sep;
+	gboolean dir;
+	gboolean ret;
+
+	dir = (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
+
 	if (data->cancel_flag && *data->cancel_flag)
-		return TRUE;
+		return FALSE;
 
 	/* skip hidden and unreadable files and directories */
-	if (g_str_has_prefix (rel_path, ".") ||
-	    ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
-	    !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE))) {
+	sep = strrchr (rel_path, G_DIR_SEPARATOR);
+	if ((sep != NULL && g_str_has_prefix (sep + 1, ".")) ||
+	    (sep == NULL && g_str_has_prefix (rel_path, ".")) ||
+	    _gnomevfs_info_unreadable (info)) {
 		*recurse = FALSE;
 		return TRUE;
 	}
 
-	if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
-		char *path, *escaped_rel_path;
+	escaped_rel_path = gnome_vfs_escape_path_string (rel_path);
+	escaped_rel_path = escape_extra_gnome_vfs_chars (escaped_rel_path);
+	path = g_build_filename (data->uri, escaped_rel_path, NULL);
+	ret = (data->func) (path, dir, data->user_data);
+	g_free (escaped_rel_path);
+	g_free (path);
 
-		escaped_rel_path = gnome_vfs_escape_path_string (rel_path);
-		escaped_rel_path = escape_extra_gnome_vfs_chars (escaped_rel_path);
-		path = g_build_filename (data->uri, escaped_rel_path, NULL);
-		(data->func) (path, data->user_data);
-		g_free (escaped_rel_path);
-		g_free (path);
-	}
-
-	*recurse = !recursing_will_loop;
+	/* if recursing will not loop, and function wants to recurse, then do so */
+	*recurse = !recursing_will_loop && ret;
 	return TRUE;
 }
 
 void
 rb_uri_handle_recursively (const char *text_uri,
-		           GFunc func,
+		           RBUriRecurseFunc func,
 			   gboolean *cancelflag,
 		           gpointer user_data)
 {
@@ -678,10 +975,11 @@ rb_uri_handle_recursively (const char *text_uri,
 	GnomeVFSFileInfoOptions flags;
 	GnomeVFSResult result;
 	
-	data->uri = text_uri;
+	data->uri = g_strdup (text_uri);
 	data->func = func;
 	data->user_data = user_data;
 	data->cancel_flag = cancelflag;
+	data->data_destroy = NULL;
 
 	flags = GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
 		GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
@@ -692,9 +990,116 @@ rb_uri_handle_recursively (const char *text_uri,
 					    GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
 					    (GnomeVFSDirectoryVisitFunc)rb_uri_handle_recursively_cb,
 					    data);
-	g_free (data);
+	_rb_uri_recurse_data_free (data);
 }
 
+/* runs in main thread */
+static gboolean
+_recurse_async_idle_cb (RBUriHandleRecursivelyAsyncData *data)
+{
+	GList *ul, *dl;
+
+	g_mutex_lock (data->results_lock);
+
+	for (ul = data->uri_results, dl = data->dir_results;
+	     ul != NULL;
+	     ul = g_list_next (ul), dl = g_list_next (dl)) {
+		g_assert (dl != NULL);
+
+		data->func ((const char*)ul->data, (GPOINTER_TO_INT (dl->data) == 1), data->user_data);
+		g_free (ul->data);
+	}
+	g_assert (dl == NULL);
+
+
+	g_list_free (data->uri_results);
+	data->uri_results = NULL;
+	g_list_free (data->dir_results);
+	data->dir_results = NULL;
+
+	data->results_idle_id = 0;
+	g_mutex_unlock (data->results_lock);
+	return FALSE;
+}
+
+/* runs in main thread */
+static gboolean
+_recurse_async_data_free (RBUriHandleRecursivelyAsyncData *data)
+{
+	if (data->results_idle_id) {
+		g_source_remove (data->results_idle_id);
+		_recurse_async_idle_cb (data); /* process last results */
+	}
+
+	g_list_free (data->uri_results);
+	data->uri_results = NULL;
+	g_list_free (data->dir_results);
+	data->dir_results = NULL;
+
+	g_mutex_free (data->results_lock);
+	_rb_uri_recurse_data_free (&data->data);
+
+	return FALSE;
+}
+
+/* runs in worker thread */
+static gboolean
+_recurse_async_cb (const char *uri, gboolean dir, RBUriHandleRecursivelyAsyncData *data)
+{
+	g_mutex_lock (data->results_lock);
+
+	data->uri_results = g_list_prepend (data->uri_results, g_strdup (uri));
+	data->dir_results = g_list_prepend (data->dir_results, GINT_TO_POINTER (dir ? 1 : 0));
+	if (data->results_idle_id == 0)
+		g_idle_add ((GSourceFunc)_recurse_async_idle_cb, data);
+
+	g_mutex_unlock (data->results_lock);
+	return TRUE;
+}
+
+static gpointer
+_recurse_async_func (RBUriHandleRecursivelyAsyncData *data)
+{
+	GnomeVFSFileInfoOptions flags;
+	GnomeVFSResult result;
+
+	flags = GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+		GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
+		GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
+		GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
+	result = gnome_vfs_directory_visit (data->data.uri,
+					    flags,
+					    GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
+					    (GnomeVFSDirectoryVisitFunc)rb_uri_handle_recursively_cb,
+					    &data->data);
+	
+	g_idle_add ((GSourceFunc)_recurse_async_data_free, data);
+	return NULL;
+}
+
+void
+rb_uri_handle_recursively_async (const char *text_uri,
+			         RBUriRecurseFunc func,
+				 gboolean *cancelflag,
+			         gpointer user_data,
+				 GDestroyNotify data_destroy)
+{
+	RBUriHandleRecursivelyAsyncData *data = g_new0 (RBUriHandleRecursivelyAsyncData, 1);
+	
+	data->data.uri = g_strdup (text_uri);
+	data->func = (RBUriRecurseFunc)_recurse_async_cb;
+	data->data.user_data = user_data;
+	data->data.cancel_flag = cancelflag;
+	data->data.data_destroy = data_destroy;
+
+	data->results_lock = g_mutex_new ();
+	data->data.func = func;
+	data->user_data = data;
+
+	g_thread_create ((GThreadFunc)_recurse_async_func, data, FALSE, NULL);
+}
+
+
 GnomeVFSResult
 rb_uri_mkstemp (const char *prefix, char **uri_ret, GnomeVFSHandle **ret)
 {
@@ -724,37 +1129,37 @@ rb_canonicalise_uri (const char *uri)
 {
 	char *result = NULL;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	if (uri[0] == '/') {
 		/* local path */
 		char *tmp;
 		result = gnome_vfs_make_path_name_canonical (uri);
-		tmp = gnome_vfs_escape_path_string (result);
+		tmp = gnome_vfs_get_uri_from_local_path (result);
 		g_free (result);
-		tmp = escape_extra_gnome_vfs_chars (tmp);
-		result = g_strconcat ("file://", tmp, NULL);
-		g_free (tmp);
-	} else  if (g_str_has_prefix (uri, "file://")) {
-	    	/* local file, rhythmdb wants this path escaped */
-		char *tmp1, *tmp2;
-		tmp1  = gnome_vfs_unescape_string (uri + 7, NULL);  /* ignore "file://" */
-		tmp2 = gnome_vfs_escape_path_string (tmp1);
-		g_free (tmp1);
-		tmp2 = escape_extra_gnome_vfs_chars (tmp2);
-		result = g_strconcat ("file://", tmp2, NULL); /* re-add scheme */
-		g_free (tmp2);
-	} else {
-		GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
+		if (tmp == NULL)
+			return NULL;
+		result = tmp;
+	} else if (strstr (uri, "://") == NULL) {
+		/* local relative path */
+		char *curdir, *escaped, *curdir_withslash;
 
-		if (vfsuri != NULL) {
-			/* non-local uri, leave as-is */
-			gnome_vfs_uri_unref (vfsuri);
+		curdir = g_get_current_dir ();
+		escaped = gnome_vfs_escape_path_string (curdir);
+		curdir_withslash = g_strdup_printf ("file://%s%c",
+						    escaped, G_DIR_SEPARATOR);
+		g_free (escaped);
+		g_free (curdir);
+
+		escaped = gnome_vfs_escape_path_string (uri);
+		result = gnome_vfs_uri_make_full_from_relative
+			(curdir_withslash, escaped);
+		g_free (curdir_withslash);
+		g_free (escaped);
+	} else {
+		result = gnome_vfs_make_uri_canonical (uri);
+		if (result == NULL)
 			result = g_strdup (uri);
-		} else {
-			/* this may just mean that gnome-vfs doesn't recognise the
-			 * uri scheme, so return it as is */
-			rb_debug ("Error processing probable URI %s", uri);
-			result = g_strdup (uri);
-		}
 	}
 
 	return result;
@@ -765,8 +1170,12 @@ rb_uri_append_path (const char *uri, const char *path)
 {
 	GnomeVFSURI *vfs_uri, *full_uri;
 	char *result;
-	
+
 	vfs_uri = gnome_vfs_uri_new (uri);
+	if (vfs_uri == NULL) {
+		return NULL;
+	}
+
 	full_uri = gnome_vfs_uri_append_path (vfs_uri, path);
 	gnome_vfs_uri_unref (vfs_uri);
 	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
@@ -780,8 +1189,19 @@ rb_uri_append_uri (const char *uri, const char *fragment)
 {
 	GnomeVFSURI *vfs_uri, *full_uri;
 	char *result;
-	
+
 	vfs_uri = gnome_vfs_uri_new (uri);
+	if (vfs_uri == NULL) {
+		return NULL;
+	}
+
+	/* skip scheme component of the fragment */
+	if (has_valid_scheme (fragment)) {
+		while (is_valid_scheme_character (*fragment))
+			fragment++;
+	}
+	fragment++;
+
 	full_uri = gnome_vfs_uri_append_string (vfs_uri, fragment);
 	gnome_vfs_uri_unref (vfs_uri);
 	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
@@ -797,9 +1217,49 @@ rb_uri_get_dir_name (const char *uri)
 	char *dirname;
 
 	vfs_uri = gnome_vfs_uri_new (uri);
+	if (vfs_uri == NULL) {
+		return NULL;
+	}
+
 	dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
 	gnome_vfs_uri_unref (vfs_uri);
 
 	return dirname;
 }
 
+char *
+rb_uri_get_short_path_name (const char *uri)
+{
+	const char *start;
+	const char *end;
+
+	if (uri == NULL)
+		return NULL;
+
+	/* skip query string */
+	end = g_utf8_strchr (uri, -1, '?');
+
+	start = g_utf8_strrchr (uri, end ? (end - uri) : -1, GNOME_VFS_URI_PATH_CHR);
+	if (start == NULL) {
+		/* no separator, just a single file name */
+	} else if ((start + 1 == end) || *(start + 1) == '\0') {
+		/* last character is the separator, so find the previous one */
+		end = start;
+		start = g_utf8_strrchr (uri, (end - uri)-1, GNOME_VFS_URI_PATH_CHR);
+
+		if (start != NULL)
+			start++;
+	} else {
+		start++;
+	}
+
+	if (start == NULL)
+		start = uri;
+
+	if (end == NULL) {
+		return g_strdup (start);
+	} else {
+		return g_strndup (start, (end - start));
+	}
+}
+
