@@ -26,6 +26,7 @@
 #include <glib.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkuimanager.h>
+#include <gtk/gtktreeview.h>
 
 G_BEGIN_DECLS
 
@@ -57,9 +58,12 @@ gchar** rb_string_split_words (const gchar *string);
 
 gboolean rb_string_list_equal (GList *a, GList *b);
 gboolean rb_string_list_contains (GList *list, const char *s);
-void rb_list_deep_free (GList *list);
 GList* rb_string_list_copy (GList *list);
 
+void rb_list_deep_free (GList *list);
+void rb_list_destroy_free (GList *list, GDestroyNotify destroyer);
+void rb_slist_deep_free (GSList *list);
+
 gboolean rb_str_in_strv (const char *needle, char **haystack);
 
 GList* rb_collate_hash_table_keys (GHashTable *table);
@@ -77,6 +81,12 @@ void rb_value_free (GValue *val); /* g_value_unset, g_slice_free */
 
 void rb_assert_locked (GMutex *mutex);
 
+void rb_set_tree_view_column_fixed_width (GtkWidget *treeview,
+					  GtkTreeViewColumn *column,
+					  GtkCellRenderer *renderer,
+					  const char **strings,
+					  int padding);
+
 G_END_DECLS
 
 #endif /* __RB_UTIL_H */
