@@ -25,19 +25,13 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
-#include <gst/gst.h>
+#include <gst/gsttagsetter.h>
+#include <gst/tag/tag.h>
 #include <gst/gsturi.h>
 
-#ifdef HAVE_GSTREAMER_0_8
-#include <gst/gsttag.h>
-#define gst_caps_unref gst_caps_free
-#define gst_tag_setter_add_tag_values gst_tag_setter_add_values
-#define gst_tag_setter_set_tag_merge_mode gst_tag_setter_set_merge_mode
-#define gst_tag_setter_merge_tags gst_tag_setter_merge
-#endif
-#ifdef HAVE_GSTREAMER_0_10
-#include <gst/gsttagsetter.h>
-#endif
+#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+#include <gst/pbutils/pbutils.h>
+#endif /* HAVE_GSTREAMER_0_10_MISSING_PLUGINS */
 
 #include "rb-metadata.h"
 #include "rb-debug.h"
@@ -50,7 +44,6 @@ static void rb_metadata_finalize (GObject *object);
 
 typedef GstElement *(*RBAddTaggerElem) (RBMetaData *, GstElement *);
 
-
 /*
  * The list of mine type prefixes for files that shouldn't display errors about being non-audio.
  * Useful for people who have cover art, et cetera, in their music directories
@@ -59,7 +52,8 @@ const char * ignore_mime_types[] = {
 	"image/",
 	"text/",
 	"application/xml",
-	"application/zip"
+	"application/zip",
+	"application/x-executable"
 };
 
 /*
@@ -79,7 +73,7 @@ struct RBMetadataGstType
 struct RBMetaDataPrivate
 {
 	char *uri;
-  
+
 	GHashTable *metadata;
 
 	GstElement *pipeline;
@@ -93,14 +87,15 @@ struct RBMetaDataPrivate
 	char *type;
 	gboolean handoff;
 	gboolean eos;
-	gboolean non_audio;
+	gboolean has_audio;
+	gboolean has_non_audio;
 	gboolean has_video;
+	GSList *missing_plugins;
 	GError *error;
 };
 
 #define RB_METADATA_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_METADATA, RBMetaDataPrivate))
 
-#ifdef HAVE_GSTREAMER_0_10
 static void
 gst_date_gulong_transform (const GValue *src, GValue *dest)
 {
@@ -119,8 +114,6 @@ gulong_gst_date_transform (const GValue *src, GValue *dest)
 	g_date_free (date);
 }
 
-#endif
-
 static void
 rb_metadata_class_init (RBMetaDataClass *klass)
 {
@@ -129,13 +122,10 @@ rb_metadata_class_init (RBMetaDataClass *klass)
 	object_class->finalize = rb_metadata_finalize;
 
 	g_type_class_add_private (klass, sizeof (RBMetaDataPrivate));
-#ifdef HAVE_GSTREAMER_0_10
 	g_value_register_transform_func (GST_TYPE_DATE, G_TYPE_ULONG, gst_date_gulong_transform);
 	g_value_register_transform_func (G_TYPE_ULONG, GST_TYPE_DATE, gulong_gst_date_transform);
-#endif
 }
 
-
 static GstElement *
 rb_add_flac_tagger (RBMetaData *md, GstElement *element)
 {
@@ -152,8 +142,6 @@ rb_add_flac_tagger (RBMetaData *md, GstElement *element)
 	return tagger;
 }
 
-
-#ifdef HAVE_GSTREAMER_0_10
 static void
 id3_pad_added_cb (GstElement *demux, GstPad *pad, GstElement *mux)
 {
@@ -167,17 +155,17 @@ id3_pad_added_cb (GstElement *demux, GstPad *pad, GstElement *mux)
 }
 
 static gboolean
-rb_gst_plugin_greater (const char *plugin, gint major, gint minor, gint micro)
+rb_gst_plugin_greater (const char *plugin, const char *element, gint major, gint minor, gint micro)
 {
 	const char *version;
 	GstPlugin *p;
 	guint i;
 	guint count;
 
-	if (gst_default_registry_check_feature_version (plugin, major, minor, micro + 1))
+	if (gst_default_registry_check_feature_version (element, major, minor, micro + 1))
 		return TRUE;
-	
-	if (!gst_default_registry_check_feature_version (plugin, major, minor, micro))
+
+	if (!gst_default_registry_check_feature_version (element, major, minor, micro))
 		return FALSE;
 
 	p = gst_default_registry_find_plugin (plugin);
@@ -202,14 +190,12 @@ rb_add_id3_tagger (RBMetaData *md, GstElement *element)
 	mux = gst_element_factory_make ("id3v2mux", NULL);
 	if (mux != NULL) {
 		/* check for backwards id3v2mux merge-mode */
-		if (!rb_gst_plugin_greater ("id3v2mux", 0, 10, 3))
-			gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (mux), GST_TAG_MERGE_REPLACE);
-	} else {
-		mux =  gst_element_factory_make ("id3mux", NULL);
-
-		/* check for backwards id3mux merge-mode */
-		if (mux && !rb_gst_plugin_greater ("mad", 0, 10, 3))
+		if (!rb_gst_plugin_greater ("taglib", "id3v2mux", 0, 10, 3)) {
+			rb_debug ("using id3v2mux with backwards merge mode");
 			gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (mux), GST_TAG_MERGE_REPLACE);
+		} else {
+			rb_debug ("using id3v2mux");
+		}
 	}
 
 	if (demux == NULL || mux == NULL)
@@ -231,7 +217,6 @@ error:
 	return NULL;
 }
 
-
 static void
 ogg_pad_added_cb (GstElement *demux, GstPad *pad, RBMetaData *md)
 {
@@ -255,7 +240,17 @@ ogg_pad_added_cb (GstElement *demux, GstPad *pad, RBMetaData *md)
 		rb_debug ("found vorbis stream in ogg container, using vorbistag");
 
 		parser = gst_element_factory_make ("vorbisparse", NULL);
+		if (parser == NULL) {
+			rb_debug ("could not create vorbisparse element");
+			goto end;
+		}
+
 		tagger = gst_element_factory_make ("vorbistag", NULL);
+		if (tagger == NULL) {
+			rb_debug ("could not create vorbistag element");
+			gst_object_unref (parser);
+			goto end;
+		}
 
 		bin = GST_BIN (gst_element_get_parent (mux));
 		gst_bin_add_many (bin, tagger, parser, NULL);
@@ -266,7 +261,7 @@ ogg_pad_added_cb (GstElement *demux, GstPad *pad, RBMetaData *md)
 		gst_element_get_state (mux, &state, NULL, 0);
 		gst_element_set_state (parser, state);
 		gst_element_set_state (tagger, state);
-	
+
 		conn_pad = gst_element_get_compatible_pad (tagger, pad, NULL);
 		gst_pad_link (pad, conn_pad);
 
@@ -277,6 +272,7 @@ ogg_pad_added_cb (GstElement *demux, GstPad *pad, RBMetaData *md)
 		rb_debug ("found stream in ogg container with no known tagging element");
 	}
 
+end:
 	gst_caps_unref (caps);
 }
 
@@ -307,41 +303,6 @@ error:
 	return NULL;
 }
 
-#elif HAVE_GSTREAMER_0_8
-static GstElement *
-rb_add_id3_tagger (RBMetaData *md, GstElement *element)
-{
-	GstElement *tagger, *identity;
-	GstCaps *filtercaps = NULL;
-
-	if (!(tagger = gst_element_factory_make ("id3tag", "tagger"))) 
-		return NULL;
-
-	if (!(identity = gst_element_factory_make ("identity", "identity"))) {
-		gst_object_unref (GST_OBJECT (tagger));
-		return NULL;
-	}
-
-	gst_bin_add_many (GST_BIN (md->priv->pipeline), tagger, identity, NULL);
-
-	filtercaps = gst_caps_new_simple ("application/x-id3", NULL);
-	if (!gst_element_link_filtered (tagger, identity, filtercaps)) {
-		g_warning ("Linking id3tag and identity failed");
-		gst_caps_free (filtercaps);
-		gst_object_unref (GST_OBJECT(tagger));
-		gst_object_unref (GST_OBJECT(identity));
-		return NULL;
-        }
-	gst_caps_free (filtercaps);
-
-	gst_element_link_many(element, tagger, NULL);
-
-	gst_tag_setter_merge_tags (GST_TAG_SETTER (tagger), md->priv->tags, GST_TAG_MERGE_REPLACE_ALL);
-
-	return identity;
-}
-#endif
-
 static void
 add_supported_type (RBMetaData *md,
 		    const char *mime,
@@ -361,35 +322,27 @@ rb_metadata_init (RBMetaData *md)
 	RBAddTaggerElem tagger;
 	gboolean has_gnomevfssink = FALSE;
 	gboolean has_id3 = FALSE;
-        
+
 	md->priv = RB_METADATA_GET_PRIVATE (md);
 
 	md->priv->supported_types = g_ptr_array_new ();
-	
+
  	/* the list of supported types serves two purposes:
  	 * - it knows how to construct elements for tag writing
  	 * - it knows human-readable names for MIME types so we can say
  	 *     "There is no plugin available to play WMV files"
  	 *     rather than " .. play video/x-ms-asf files".
  	 *
- 	 * only registering types we have plugins for defeats the second 
+ 	 * only registering types we have plugins for defeats the second
 	 * purpose.
  	 */
 	has_gnomevfssink = (gst_element_factory_find ("gnomevfssrc") != NULL &&
 			    gst_element_factory_find ("gnomevfssink") != NULL);
-#ifdef HAVE_GSTREAMER_0_10
-	has_id3 = (gst_element_factory_find ("id3demux") != NULL) &&
-			(gst_default_registry_check_feature_version ("id3mux", 0, 10, 2) ||
-			 (gst_element_factory_find ("id3v2mux") != NULL));
-#elif HAVE_GSTREAMER_0_8
-	has_id3 = (gst_element_factory_find ("id3tag") != NULL);
-#endif
-
+	has_id3 = (gst_element_factory_find ("id3v2mux") != NULL);
 	tagger = (has_gnomevfssink && has_id3) ?  rb_add_id3_tagger : NULL;
 	add_supported_type (md, "application/x-id3", tagger, "MP3");
 	add_supported_type (md, "audio/mpeg", tagger, "MP3");
 
-#ifdef HAVE_GSTREAMER_0_10
 	{
 		gboolean has_vorbis;
 
@@ -399,9 +352,6 @@ rb_metadata_init (RBMetaData *md)
 			      gst_default_registry_check_feature_version ("oggdemux", 0, 10, 6));
 		tagger = (has_gnomevfssink && has_vorbis) ?  rb_add_ogg_tagger : NULL;
 	}
-#else
-	tagger = NULL;
-#endif
 	add_supported_type (md, "application/ogg", tagger, "Ogg Vorbis");
 	add_supported_type (md, "audio/x-vorbis", tagger, "Ogg Vorbis");
 
@@ -473,7 +423,7 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_COMMENT;
 	else if (!strcmp (tag, GST_TAG_TRACK_NUMBER))
 		return RB_METADATA_FIELD_TRACK_NUMBER;
-	else if (!strcmp (tag, GST_TAG_TRACK_COUNT)) 
+	else if (!strcmp (tag, GST_TAG_TRACK_COUNT))
 		return RB_METADATA_FIELD_MAX_TRACK_NUMBER;
 	else if (!strcmp (tag, GST_TAG_ALBUM_VOLUME_NUMBER))
 		return RB_METADATA_FIELD_DISC_NUMBER;
@@ -509,6 +459,19 @@ rb_metadata_gst_tag_to_field (const char *tag)
 		return RB_METADATA_FIELD_ALBUM_GAIN;
 	else if (!strcmp (tag, GST_TAG_ALBUM_PEAK))
 		return RB_METADATA_FIELD_ALBUM_PEAK;
+#ifdef GST_TAG_MUSICBRAINZ_TRACKID
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID))
+		return RB_METADATA_FIELD_MUSICBRAINZ_TRACKID;
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_ARTISTID))
+		return RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID;
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_ALBUMID))
+		return RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID;
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_ALBUMARTISTID))
+		return RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID;
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_SORTNAME))
+		return RB_METADATA_FIELD_ARTIST_SORTNAME;
+
+#endif
 	else
 		return -1;
 }
@@ -568,6 +531,18 @@ rb_metadata_gst_field_to_gst_tag (RBMetaDataField field)
 		return GST_TAG_ALBUM_GAIN;
 	case RB_METADATA_FIELD_ALBUM_PEAK:
 		return GST_TAG_ALBUM_PEAK;
+#ifdef GST_TAG_MUSICBRAINZ_TRACKID
+	case RB_METADATA_FIELD_MUSICBRAINZ_TRACKID:
+		return GST_TAG_MUSICBRAINZ_TRACKID;
+	case RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID:
+		return GST_TAG_MUSICBRAINZ_ARTISTID;
+	case RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID:
+		return GST_TAG_MUSICBRAINZ_ALBUMID;
+	case RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID:
+		return GST_TAG_MUSICBRAINZ_ALBUMARTISTID;
+	case RB_METADATA_FIELD_ARTIST_SORTNAME:
+		return GST_TAG_MUSICBRAINZ_SORTNAME;
+#endif
 	default:
 		return NULL;
 	}
@@ -605,7 +580,7 @@ make_undecodable_error (RBMetaData *md)
 	human_name= rb_metadata_gst_type_to_name (md, md->priv->type);
 	if (human_name == NULL)
 		human_name = rb_mime_get_friendly_name (md->priv->type);
-	
+
 	if (human_name) {
 		return g_strdup_printf (_("The GStreamer plugins to decode \"%s\" files cannot be found"),
 					human_name);
@@ -639,9 +614,9 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 	newval = g_new0 (GValue, 1);
 	g_value_init (newval, type);
 	if (!g_value_transform (val, newval)) {
-		
+
 		rb_debug ("Could not transform tag value type %s into %s",
-			  g_type_name (G_VALUE_TYPE (val)), 
+			  g_type_name (G_VALUE_TYPE (val)),
 			  g_type_name (G_VALUE_TYPE (newval)));
 		g_value_unset (newval);
 		g_free (newval);
@@ -650,10 +625,11 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 
 	switch (type) {
 	case G_TYPE_STRING: {
-		/* Reject invalid utf-8 strings, and then
-		 * remove leading and trailing whitespace.
-		 */ 
+		/* Reject invalid utf-8 strings, shorter duplicated tags
+		 * and then remove leading and trailing whitespace.
+		 */
 		char *str;
+
 		str = g_value_dup_string (newval);
 
 		if (!g_utf8_validate (str, -1, NULL)) {
@@ -664,6 +640,25 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 			return;
 		}
 		str = g_strstrip (str);
+
+		/* Check whether we have a shorter duplicate tag,
+		 * Doesn't work with non-normalised UTF-8 strings */
+		val = g_hash_table_lookup (md->priv->metadata,
+					   GINT_TO_POINTER (field));
+		if (val != NULL) {
+			const char *old_str;
+			old_str = g_value_get_string (val);
+			if (old_str != NULL
+			    && g_utf8_strlen (old_str, -1) > g_utf8_strlen (str, -1)) {
+				if (g_str_has_prefix (old_str, str) != FALSE) {
+					rb_debug ("Got shorter duplicate tag");
+					g_free (str);
+					g_value_unset (newval);
+					g_free (newval);
+					return;
+				}
+			}
+		}
 		g_value_take_string (newval, str);
 		break;
 	}
@@ -681,7 +676,7 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 	}
 
 	case RB_METADATA_FIELD_DURATION: {
-		/* GStreamer sends us duration in ns, 
+		/* GStreamer sends us duration in ns,
 		 * but we need it in seconds
 		 */
 		guint64 duration;
@@ -694,67 +689,11 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 		break;
 	}
 
-	g_hash_table_insert (md->priv->metadata, 
+	g_hash_table_insert (md->priv->metadata,
 			     GINT_TO_POINTER (field),
 			     newval);
 }
 
-#ifdef HAVE_GSTREAMER_0_8
-static void
-rb_metadata_gst_found_tag (GObject *pipeline, GstElement *source, GstTagList *tags, RBMetaData *md)
-{
-	gst_tag_list_foreach (tags, (GstTagForeachFunc) rb_metadata_gst_load_tag, md);
-}
-
-static void
-rb_metadata_gst_fakesink_handoff_cb (GstElement *fakesink, GstBuffer *buf, GstPad *pad, RBMetaData *md)
-{
-	if (md->priv->handoff) {
-		/* we get lots of these with decodebin.  why? */
-		/* rb_debug ("caught recursive handoff!"); */
-		return;
-	} else if (md->priv->eos) {
-		rb_debug ("caught handoff after eos!");
-		return;
-	}
-		
-	rb_debug ("in fakesink handoff");
-	md->priv->handoff = TRUE;
-}
-
-static void
-rb_metadata_gst_error_cb (GstElement *element,
-			  GstElement *source,
-			  GError *error,
-			  gchar *debug,
-			  RBMetaData *md)
-{
-	if (error->domain == GST_STREAM_ERROR &&
-	    error->code == GST_STREAM_ERROR_TYPE_NOT_FOUND) {
-		rb_debug ("caught type not found error");
-
-	} else {
-		rb_debug ("caught error: %s ", error->message);
-		md->priv->error = g_error_new_literal (RB_METADATA_ERROR,
-						       RB_METADATA_ERROR_GENERAL,
-						       error->message);
-	}
-}
-
-static void
-rb_metadata_gst_eos_cb (GstElement *element, RBMetaData *md)
-{
-	rb_debug ("caught eos");
-	if (md->priv->eos) {
-		rb_debug ("RENTERED EOS!");
-		return;
-	}
-	gst_element_set_state (md->priv->sink, GST_STATE_NULL);
-	md->priv->eos = TRUE;
-}
-
-#endif
-
 static void
 rb_metadata_gst_typefind_cb (GstElement *typefind, guint probability, GstCaps *caps, RBMetaData *md)
 {
@@ -767,7 +706,6 @@ rb_metadata_gst_typefind_cb (GstElement *typefind, guint probability, GstCaps *c
 	g_signal_handler_disconnect (typefind, md->priv->typefind_cb_id);
 }
 
-
 static void
 rb_metadata_gst_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean last, RBMetaData *md)
 {
@@ -781,7 +719,7 @@ rb_metadata_gst_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean
 	/* we get "ANY" caps for text/plain files etc. */
 	if (gst_caps_is_empty (caps) || gst_caps_is_any (caps)) {
 		rb_debug ("decoded pad with no caps or any caps.  this file is boring.");
-		md->priv->non_audio = TRUE;
+		md->priv->has_non_audio = TRUE;
 		cancel = TRUE;
 	} else {
 		GstPad *sink_pad;
@@ -795,40 +733,33 @@ rb_metadata_gst_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean
 
 		if (g_str_has_prefix (mimetype, "audio/x-raw")) {
 			rb_debug ("got decoded audio pad of type %s", mimetype);
+			md->priv->has_audio = TRUE;
 		} else if (g_str_has_prefix (mimetype, "video/")) {
 			rb_debug ("got decoded video pad of type %s", mimetype);
-			md->priv->non_audio = TRUE;
+			md->priv->has_non_audio = TRUE;
 			md->priv->has_video = TRUE;
 		} else {
 			/* assume anything we can get a video or text stream out of is
 			 * something that should be fed to totem rather than rhythmbox.
 			 */
 			rb_debug ("got decoded pad of non-audio type %s", mimetype);
-			md->priv->non_audio = TRUE;
+			md->priv->has_non_audio = TRUE;
 		}
 	}
 
 	gst_caps_unref (caps);
 
-#ifdef HAVE_GSTREAMER_0_10
-	/* if this is non-audio, cancel the operation
-	 * under 0.8 this causes assertion faliures when a pad with no caps in found
-	 * it isn't /needed/ under 0.8, so we don't do it.
-	 *
-	 * This seems to cause some deadlocks with video files, so only do it 
+	/* If this is non-audio, cancel the operation.
+	 * This seems to cause some deadlocks with video files, so only do it
 	 * when we get no/any caps.
 	 */
 	if (cancel)
 		gst_element_set_state (md->priv->pipeline, GST_STATE_NULL);
-#endif
 }
 
 static void
 rb_metadata_gst_unknown_type_cb (GstElement *decodebin, GstPad *pad, GstCaps *caps, RBMetaData *md)
 {
-	/* try to shortcut it a bit */
-	md->priv->non_audio = TRUE;
-
 	if (!gst_caps_is_empty (caps) && !gst_caps_is_any (caps)) {
 		GstStructure *structure;
 		const gchar *mimetype;
@@ -838,13 +769,15 @@ rb_metadata_gst_unknown_type_cb (GstElement *decodebin, GstPad *pad, GstCaps *ca
 
 		g_free (md->priv->type);
 		md->priv->type = g_strdup (mimetype);
-		
+
 		rb_debug ("decodebin emitted unknown type signal for %s", mimetype);
 	} else {
 		rb_debug ("decodebin emitted unknown type signal");
 	}
 
-#ifdef HAVE_GSTREAMER_0_10
+	md->priv->has_non_audio = TRUE;
+#ifndef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+	/* try to shortcut it a bit */
 	{
 		char *msg;
 
@@ -871,7 +804,17 @@ static GstElement *make_pipeline_element (GstElement *pipeline, const char *elem
 	return elem;
 }
 
-#ifdef HAVE_GSTREAMER_0_10
+#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+static void
+rb_metadata_handle_missing_plugin_message (RBMetaData *md, GstMessage *message)
+{
+	rb_debug ("got missing-plugin message from %s: %s",
+		  GST_OBJECT_NAME (GST_MESSAGE_SRC (message)),
+		  gst_missing_plugin_message_get_installer_detail (message));
+	md->priv->missing_plugins = g_slist_prepend (md->priv->missing_plugins, gst_message_ref (message));
+}
+#endif
+
 static gboolean
 rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
 {
@@ -879,30 +822,45 @@ rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
 	case GST_MESSAGE_EOS:
 		rb_debug ("EOS reached");
 		md->priv->eos = TRUE;
-		break;
+		return TRUE;
+
 	case GST_MESSAGE_ERROR:
 	{
 		GError *gerror;
 		gchar *debug;
+		char *src;
+
+		src = gst_element_get_name (GST_MESSAGE_SRC (message));
+		rb_debug ("got error message from %s", src);
+		g_free (src);
 
 		gst_message_parse_error (message, &gerror, &debug);
 		if (gerror->domain == GST_STREAM_ERROR &&
 		    gerror->code == GST_STREAM_ERROR_TYPE_NOT_FOUND) {
 			rb_debug ("caught type not found error");
+		} else if (gerror->domain == GST_STREAM_ERROR &&
+			   gerror->code == GST_STREAM_ERROR_WRONG_TYPE &&
+			   md->priv->type != NULL &&
+			   strcmp (md->priv->type, "text/plain") == 0) {
+			rb_debug ("got WRONG_TYPE error for text/plain: setting non-audio flag");
+			md->priv->has_non_audio = TRUE;
 		} else if (md->priv->error) {
 			rb_debug ("caught error: %s, but we've already got one", gerror->message);
 		} else {
 			rb_debug ("caught error: %s ", gerror->message);
-			
+
 			g_clear_error (&md->priv->error);
 			md->priv->error = g_error_new_literal (RB_METADATA_ERROR,
 							       RB_METADATA_ERROR_GENERAL,
 							       gerror->message);
 		}
 
+		/* treat this as equivalent to EOS */
+		md->priv->eos = TRUE;
+
 		g_error_free (gerror);
 		g_free (debug);
-		break;
+		return TRUE;
 	}
 	case GST_MESSAGE_TAG:
 	{
@@ -922,15 +880,31 @@ rb_metadata_bus_handler (GstBus *bus, GstMessage *message, RBMetaData *md)
 		}
 		break;
 	}
-	default:
-		rb_debug ("message of type %d", GST_MESSAGE_TYPE (message));
+	case GST_MESSAGE_ELEMENT:
+	{
+#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+		if (gst_is_missing_plugin_message (message)) {
+			rb_metadata_handle_missing_plugin_message (md, message);
+		}
+#endif
 		break;
 	}
+	default:
+	{
+		char *src;
+
+		src = gst_element_get_name (GST_MESSAGE_SRC (message));
+		rb_debug ("message of type %s from %s",
+			  GST_MESSAGE_TYPE_NAME (message), src);
+		g_free (src);
+		break;
+	}
+	}
 
 	return FALSE;
 }
 
-static void 
+static void
 rb_metadata_event_loop (RBMetaData *md, GstElement *element, gboolean block)
 {
 	GstBus *bus;
@@ -957,7 +931,6 @@ rb_metadata_event_loop (RBMetaData *md, GstElement *element, gboolean block)
 	}
 	gst_object_unref (bus);
 }
-#endif
 
 void
 rb_metadata_load (RBMetaData *md,
@@ -970,10 +943,9 @@ rb_metadata_load (RBMetaData *md,
 	GstElement *typefind = NULL;
 	gint64 file_size = -1;
 	GstFormat file_size_format = GST_FORMAT_BYTES;
-#ifdef HAVE_GSTREAMER_0_10
 	GstStateChangeReturn state_ret;
 	int change_timeout;
-#endif
+	GstBus *bus;
 
 	g_free (md->priv->uri);
 	md->priv->uri = NULL;
@@ -982,8 +954,10 @@ rb_metadata_load (RBMetaData *md,
 	md->priv->error = NULL;
 	md->priv->eos = FALSE;
 	md->priv->handoff = FALSE;
-	md->priv->non_audio = FALSE;
+	md->priv->has_audio = FALSE;
+	md->priv->has_non_audio = FALSE;
 	md->priv->has_video = FALSE;
+	md->priv->missing_plugins = NULL;
 
 	if (md->priv->pipeline) {
 		gst_object_unref (GST_OBJECT (md->priv->pipeline));
@@ -1030,63 +1004,53 @@ rb_metadata_load (RBMetaData *md,
  		goto out;
  	}
 
-#ifdef HAVE_GSTREAMER_0_8
-	g_object_set (G_OBJECT (md->priv->sink), "signal-handoffs", TRUE, NULL);
- 
- 	g_signal_connect_object (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md, 0);
- 	g_signal_connect_object (pipeline, "found-tag", G_CALLBACK (rb_metadata_gst_found_tag), md, 0);
-
-	g_signal_connect_object (md->priv->sink, "handoff", G_CALLBACK (rb_metadata_gst_fakesink_handoff_cb), md, 0);
-	g_signal_connect_object (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md, 0);
-#endif
  	g_signal_connect_object (decodebin, "new-decoded-pad", G_CALLBACK (rb_metadata_gst_new_decoded_pad_cb), md, 0);
  	g_signal_connect_object (decodebin, "unknown-type", G_CALLBACK (rb_metadata_gst_unknown_type_cb), md, 0);
-  
+
  	/* locate the decodebin's typefind, so we can get the have_type signal too.
- 	 * this is kind of nasty, since it relies on an essentially arbitrary string 
+ 	 * this is kind of nasty, since it relies on an essentially arbitrary string
  	 * in the decodebin code not changing.  the alternative is to have our own
  	 * typefind instance before the decodebin.  it might not like that.
  	 */
  	typefind = gst_bin_get_by_name (GST_BIN (decodebin), "typefind");
  	g_assert (typefind != NULL);
-	md->priv->typefind_cb_id = g_signal_connect_object (typefind, 
-							    "have_type", 
-							    G_CALLBACK (rb_metadata_gst_typefind_cb), 
-							    md, 
+	md->priv->typefind_cb_id = g_signal_connect_object (typefind,
+							    "have_type",
+							    G_CALLBACK (rb_metadata_gst_typefind_cb),
+							    md,
 							    0);
 	gst_object_unref (GST_OBJECT (typefind));
- 
+
  	gst_element_link (urisrc, decodebin);
 
 	md->priv->pipeline = pipeline;
-#ifdef HAVE_GSTREAMER_0_8
-	gst_element_set_state (pipeline, GST_STATE_PLAYING);
-	while (gst_bin_iterate (GST_BIN (pipeline))
-	       && md->priv->error == NULL
-	       && !md->priv->handoff
-	       && !md->priv->eos)
-		;
-
-	if (md->priv->handoff) {
-#endif
-#ifdef HAVE_GSTREAMER_0_10
 	rb_debug ("going to PAUSED for metadata, uri: %s", uri);
 	state_ret = gst_element_set_state (pipeline, GST_STATE_PAUSED);
-	change_timeout = 5;
-        while (state_ret == GST_STATE_CHANGE_ASYNC &&
+	bus = gst_element_get_bus (GST_ELEMENT (pipeline));
+	change_timeout = 0;
+	while (state_ret == GST_STATE_CHANGE_ASYNC &&
 	       !md->priv->eos &&
-	       !md->priv->non_audio &&
-	       change_timeout > 0) {
-	    GstState state;
-	    rb_debug ("element state changing asynchronously: %d, %d", state_ret, state);
-	    state_ret = gst_element_get_state (GST_ELEMENT (pipeline),
-			    &state, NULL, 1 * GST_SECOND);
-	    change_timeout--;
+	       change_timeout < 5) {
+		GstMessage *msg;
+
+		msg = gst_bus_poll (bus, GST_MESSAGE_ANY, 1 * GST_SECOND);
+		if (msg) {
+			rb_metadata_bus_handler (bus, msg, md);
+			gst_message_unref (msg);
+			change_timeout = 0;
+		} else {
+			change_timeout++;
+		}
+
+		state_ret = gst_element_get_state (pipeline, NULL, NULL, 0);
 	}
+	gst_object_unref (GST_OBJECT (bus));
+
+	rb_metadata_event_loop (md, GST_ELEMENT (pipeline), FALSE);
+
 	if (state_ret != GST_STATE_CHANGE_SUCCESS) {
 		rb_debug ("failed to go to PAUSED for %s", uri);
-		rb_metadata_event_loop (md, GST_ELEMENT (pipeline), FALSE);
-		if (!md->priv->non_audio && md->priv->error == NULL)
+		if (!md->priv->has_non_audio && md->priv->error == NULL)
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
@@ -1095,20 +1059,8 @@ rb_metadata_load (RBMetaData *md,
 		rb_debug ("gone to PAUSED for %s", uri);
 
 	if (state_ret == GST_STATE_CHANGE_SUCCESS) {
-		/* Post application specific message so we'll know when to stop
-		 * the message loop */
-		GstBus *bus;
-		bus = gst_element_get_bus (GST_ELEMENT (pipeline));
-		if (bus) {
-			gst_bus_post (bus, 
-			gst_message_new_application (GST_OBJECT (pipeline), NULL));
-			gst_object_unref (bus);
-		}
 
-		/* Poll the bus for messages */
-		rb_metadata_event_loop (md, GST_ELEMENT (pipeline), FALSE);
-#endif
-		/* We caught the first buffer(0.8) or went to PAUSED (0.10),
+		/* The pipeline went to PAUSED,
 		 * which means the decoder should have read all
 		 * of the metadata, and should know the length now.
 		 */
@@ -1116,18 +1068,13 @@ rb_metadata_load (RBMetaData *md,
 			GstFormat format = GST_FORMAT_TIME;
 			gint64 length;
 			GValue *newval;
-			
-#ifdef HAVE_GSTREAMER_0_8
-			if (gst_element_query (md->priv->sink, GST_QUERY_TOTAL, &format, &length)) {
-#endif
-#ifdef HAVE_GSTREAMER_0_10
+
 			if (gst_element_query_duration (md->priv->sink, &format, &length)) {
 				g_assert (format == GST_FORMAT_TIME);
-#endif
 				newval = g_new0 (GValue, 1);
-				
+
 				rb_debug ("duration query succeeded");
-				
+
 				g_value_init (newval, G_TYPE_ULONG);
 				/* FIXME - use guint64 for duration? */
 				g_value_set_ulong (newval, (long) (length / GST_SECOND));
@@ -1138,35 +1085,25 @@ rb_metadata_load (RBMetaData *md,
 			}
 		}
 	}
-#ifdef HAVE_GSTREAMER_0_8
-	else if (md->priv->eos) {
-		g_warning ("caught eos without handoff!");
-	}
-#endif
 
 	/* Get the file size, since it might be interesting, and return
 	 * the pipeline to NULL state.
 	 */
-#ifdef HAVE_GSTREAMER_0_8
-	gst_element_query (urisrc, GST_QUERY_TOTAL, &file_size_format, &file_size);
-	gst_element_set_state (pipeline, GST_STATE_NULL);
-#endif
-#ifdef HAVE_GSTREAMER_0_10
 	if (gst_element_query_duration (urisrc, &file_size_format, &file_size))
 		g_assert (file_size_format == GST_FORMAT_BYTES);
-	
+
 	state_ret = gst_element_set_state (pipeline, GST_STATE_NULL);
 	if (state_ret == GST_STATE_CHANGE_ASYNC) {
 		g_warning ("Failed to return metadata reader to NULL state");
 	}
 	md->priv->handoff = (state_ret == GST_STATE_CHANGE_SUCCESS);
-#endif
 
 	/* report errors for various failure cases.
 	 * these don't include the URI as the import errors source
 	 * already displays it.
 	 */
-	if (md->priv->non_audio || !md->priv->handoff) {
+	if ((md->priv->has_video || !md->priv->has_audio) &&
+	    (md->priv->has_non_audio || !md->priv->handoff)) {
 		gboolean ignore = FALSE;
 		int i;
 
@@ -1199,16 +1136,24 @@ rb_metadata_load (RBMetaData *md,
 				     RB_METADATA_ERROR_NOT_AUDIO_IGNORE,
 				     " ");
 		}
-	} else if (md->priv->error) {
+	} else if (md->priv->error != NULL) {
 		g_propagate_error (error, md->priv->error);
+		md->priv->error = NULL;
 	} else if (!md->priv->type) {
 		/* ignore really small files that can't be identified */
 		gint error_code = RB_METADATA_ERROR_UNRECOGNIZED;
 		if (file_size > 0 && file_size < REALLY_SMALL_FILE_SIZE) {
 			rb_debug ("ignoring %s because it's too small to care about", md->priv->uri);
 			error_code = RB_METADATA_ERROR_NOT_AUDIO_IGNORE;
+		} else if (file_size == 0) {
+			g_clear_error (error);
+			g_set_error (error,
+				     RB_METADATA_ERROR,
+				     RB_METADATA_ERROR_EMPTY_FILE,
+				     _("Empty file"));
+			goto out;
 		}
-		
+
 		g_clear_error (error);
 		g_set_error (error,
 			     RB_METADATA_ERROR,
@@ -1217,7 +1162,7 @@ rb_metadata_load (RBMetaData *md,
 	} else {
 		/* yay, it worked */
 		rb_debug ("successfully read metadata for %s", uri);
-	
+
 		/* it doesn't matter if we don't recognise the format,
 		 * as long as gstreamer can play it, we can put it in
 		 * the library.
@@ -1239,7 +1184,7 @@ rb_metadata_can_save (RBMetaData *md, const char *mimetype)
 {
 #ifdef ENABLE_TAG_WRITING
 	return rb_metadata_gst_type_to_tag_function (md, mimetype) != NULL;
-#else 
+#else
 	return FALSE;
 #endif
 }
@@ -1276,7 +1221,6 @@ rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, RBMetaData *md)
 	}
 }
 
-
 static gboolean
 rb_metadata_file_valid (char *original, char *newfile)
 {
@@ -1289,7 +1233,7 @@ rb_metadata_file_valid (char *original, char *newfile)
 
 	/* TODO: check that the tags are correct? */
 
-	if (error != NULL)	
+	if (error != NULL)
 		g_error_free (error);
 	g_object_unref (G_OBJECT (md));
 	return ret;
@@ -1300,8 +1244,9 @@ rb_metadata_save (RBMetaData *md, GError **error)
 {
 	GstElement *pipeline = NULL;
 	GstElement *gnomevfssrc = NULL;
-        GstElement *retag_end = NULL; /* the last elemet after retagging subpipeline */
+        GstElement *retag_end = NULL; /* the last element after retagging subpipeline */
 	const char *plugin_name = NULL;
+	char *tmpname_prefix = NULL;
 	char *tmpname = NULL;
 	GnomeVFSHandle *handle = NULL;
 	GnomeVFSResult result;
@@ -1312,21 +1257,22 @@ rb_metadata_save (RBMetaData *md, GError **error)
 
 	rb_debug ("saving metadata for uri: %s", md->priv->uri);
 
-	if ((result = rb_uri_mkstemp (md->priv->uri, &tmpname, &handle)) != GNOME_VFS_OK)
+	tmpname_prefix = rb_uri_make_hidden (md->priv->uri);
+	rb_debug ("temporary file name prefix: %s", tmpname_prefix);
+
+	result = rb_uri_mkstemp (tmpname_prefix, &tmpname, &handle);
+	g_free (tmpname_prefix);
+	if (result != GNOME_VFS_OK)
 		goto vfs_error;
 
 	pipeline = gst_pipeline_new ("pipeline");
 	md->priv->pipeline = pipeline;
 
-#ifdef HAVE_GSTREAMER_0_8
-	g_signal_connect_object (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md, 0);
-#endif
-
 	/* Source */
 	plugin_name = "gnomevfssrc";
 	if (!(gnomevfssrc = gst_element_factory_make (plugin_name, plugin_name)))
 		goto missing_plugin;
-	gst_bin_add (GST_BIN (pipeline), gnomevfssrc);		  
+	gst_bin_add (GST_BIN (pipeline), gnomevfssrc);
 	g_object_set (G_OBJECT (gnomevfssrc), "location", md->priv->uri, NULL);
 
 	/* Sink */
@@ -1335,23 +1281,20 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		goto missing_plugin;
 
 	g_object_set (G_OBJECT (md->priv->sink), "handle", handle, NULL);
-#if HAVE_GSTREAMER_0_8
-	g_signal_connect_object (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md, 0);
-#endif
 
 	md->priv->tags = gst_tag_list_new ();
-	g_hash_table_foreach (md->priv->metadata, 
+	g_hash_table_foreach (md->priv->metadata,
 			      (GHFunc) rb_metadata_gst_add_tag_data,
 			      md);
 
 	/* Tagger element(s) */
 	add_tagger_func = rb_metadata_gst_type_to_tag_function (md, md->priv->type);
-	
+
 	if (!add_tagger_func) {
 		g_set_error (error,
 			     RB_METADATA_ERROR,
 			     RB_METADATA_ERROR_UNSUPPORTED,
-			     "Unsupported file type: %s", md->priv->type);
+			     _("Unsupported file type: %s"), md->priv->type);
 		goto out_error;
 	}
 
@@ -1360,34 +1303,25 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		g_set_error (error,
 			     RB_METADATA_ERROR,
 			     RB_METADATA_ERROR_UNSUPPORTED,
-			     "Unable to create tag-writing elements");
+			     _("Unable to create tag-writing elements"));
 		goto out_error;
 	}
 
-	gst_bin_add (GST_BIN (pipeline), md->priv->sink); 
+	gst_bin_add (GST_BIN (pipeline), md->priv->sink);
 	gst_element_link_many (retag_end, md->priv->sink, NULL);
 
 	gst_element_set_state (pipeline, GST_STATE_PLAYING);
 
-#ifdef HAVE_GSTREAMER_0_8
-	while (gst_bin_iterate (GST_BIN (pipeline))
-	       && md->priv->error == NULL
-	       && !md->priv->eos)
-		;
-	gst_element_set_state (pipeline, GST_STATE_NULL);
-#endif
-#ifdef HAVE_GSTREAMER_0_10
 	rb_metadata_event_loop (md, GST_ELEMENT (pipeline), TRUE);
 	if (gst_element_set_state (pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_ASYNC) {
 		if (gst_element_get_state (pipeline, NULL, NULL, 3 * GST_SECOND) != GST_STATE_CHANGE_SUCCESS) {
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
-				     "Timeout while setting pipeline to NULL");
+				     _("Timeout while setting pipeline to NULL"));
 			goto out_error;
 		}
 	}
-#endif
 
 	if (md->priv->error) {
 		g_propagate_error (error, md->priv->error);
@@ -1403,10 +1337,10 @@ rb_metadata_save (RBMetaData *md, GError **error)
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
-				     "File corrupted during write");
+				     _("File corrupted during write"));
 			goto out_error;
 		}
-		
+
 		if ((result = gnome_vfs_move (tmpname, md->priv->uri, TRUE)) != GNOME_VFS_OK)
 				goto vfs_error;
 	}
@@ -1417,14 +1351,14 @@ vfs_error:
 		     RB_METADATA_ERROR,
 		     RB_METADATA_ERROR_GNOMEVFS,
 		     "%s",
-		     gnome_vfs_result_to_string (result)); 
+		     gnome_vfs_result_to_string (result));
 	goto out_error;
 missing_plugin:
 	g_set_error (error,
 		     RB_METADATA_ERROR,
 		     RB_METADATA_ERROR_MISSING_PLUGIN,
 		     _("Failed to create %s element; check your installation"),
-		     plugin_name); 
+		     plugin_name);
 out_error:
 	if (handle != NULL)
 		gnome_vfs_close (handle);
@@ -1467,7 +1401,7 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 {
 	GValue *newval;
 	GType type;
-	
+
 	if (rb_metadata_gst_field_to_gst_tag (field) == NULL) {
 		return FALSE;
 	}
@@ -1484,3 +1418,58 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 	return TRUE;
 }
 
+gboolean
+rb_metadata_has_missing_plugins (RBMetaData *md)
+{
+#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+	return (g_slist_length (md->priv->missing_plugins) > 0);
+#else
+	return FALSE;
+#endif
+}
+
+gboolean
+rb_metadata_get_missing_plugins (RBMetaData *md,
+				 char ***missing_plugins,
+				 char ***plugin_descriptions)
+{
+#ifdef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
+	char **mp;
+	char **pd;
+	int count;
+	int i;
+	GSList *t;
+
+	count = g_slist_length (md->priv->missing_plugins);
+	if (count == 0) {
+		return FALSE;
+	}
+
+	mp = g_new0 (char *, count + 1);
+	pd = g_new0 (char *, count + 1);
+	i = 0;
+	for (t = md->priv->missing_plugins; t != NULL; t = t->next) {
+		GstMessage *msg = GST_MESSAGE (t->data);
+		char *detail;
+		char *description;
+
+		detail = gst_missing_plugin_message_get_installer_detail (msg);
+		description = gst_missing_plugin_message_get_description (msg);
+		rb_debug ("adding [%s,%s] to return data", detail, description);
+		mp[i] = g_strdup (detail);
+		pd[i] = g_strdup (description);
+		i++;
+
+		gst_message_unref (msg);
+	}
+	g_slist_free (md->priv->missing_plugins);
+	md->priv->missing_plugins = NULL;
+
+	*missing_plugins = mp;
+	*plugin_descriptions = pd;
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
