@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -55,6 +55,14 @@ typedef enum
 	RB_METADATA_FIELD_TRACK_PEAK,		   /* double */
 	RB_METADATA_FIELD_ALBUM_GAIN,		   /* double */
 	RB_METADATA_FIELD_ALBUM_PEAK,		   /* double */
+	RB_METADATA_FIELD_LANGUAGE_CODE,	   /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_TRACKID,     /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID,    /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID,     /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID,   /* string */
+	RB_METADATA_FIELD_ARTIST_SORTNAME,         /* string */
+
+	RB_METADATA_FIELD_LAST			   /* nothing */
 } RBMetaDataField;
 
 typedef enum
@@ -67,6 +75,7 @@ typedef enum
 	RB_METADATA_ERROR_INTERNAL,
 	RB_METADATA_ERROR_NOT_AUDIO,
 	RB_METADATA_ERROR_NOT_AUDIO_IGNORE,
+	RB_METADATA_ERROR_EMPTY_FILE
 } RBMetaDataError;
 
 #define RB_METADATA_ERROR rb_metadata_error_quark ()
@@ -96,9 +105,11 @@ typedef struct
 
 GType		rb_metadata_get_type	(void);
 
-RBMetaData *	rb_metadata_new		(void);
+GType		rb_metadata_get_field_type (RBMetaDataField field);
 
-GType		rb_metadata_get_field_type (RBMetaData *md, RBMetaDataField field);
+const char *	rb_metadata_get_field_name (RBMetaDataField field);
+
+RBMetaData *	rb_metadata_new		(void);
 
 gboolean	rb_metadata_can_save	(RBMetaData *md, const char *mimetype);
 
@@ -111,6 +122,12 @@ void		rb_metadata_save	(RBMetaData *md,
 
 const char *	rb_metadata_get_mime	(RBMetaData *md);
 
+gboolean	rb_metadata_has_missing_plugins (RBMetaData *md);
+
+gboolean	rb_metadata_get_missing_plugins (RBMetaData *md,
+					 char ***missing_plugins,
+					 char ***plugin_descriptions);
+
 gboolean	rb_metadata_get		(RBMetaData *md, RBMetaDataField field,
 					 GValue *val);
 
