@@ -36,10 +36,10 @@ class AmazonCoverArtSearch (object):
 		self.cancel = False
 		self.loader = loader
 		self._supportedLocales = {
-			"en_US" : ("us", "xml.amazon.com"),
-			"en_GB" : ("uk", "xml-eu.amazon.com"),
-			"de" : ("de", "xml-eu.amazon.com"),
-			"ja" : ("jp", "xml.amazon.co.jp")
+			"en_US" : ("us", "xml.amazon.com", "music"),
+			"en_GB" : ("uk", "xml-eu.amazon.com", "music"),
+			"de" : ("de", "xml-eu.amazon.com", "music"),
+			"ja" : ("jp", "xml.amazon.co.jp", "music-jp")
 		}
 		self.db = None
 		self.entry = None
@@ -50,10 +50,15 @@ class AmazonCoverArtSearch (object):
 		if default[0] is not None:
 			if self._supportedLocales.has_key (default[0]):
 				lc_id = default[0]
+			else:
+				lang = default[0].split("_")[0]
+				if self._supportedLocales.has_key (lang):
+					lc_id = lang
 
 		lc_host = self._supportedLocales[lc_id][1]
 		lc_name = self._supportedLocales[lc_id][0]
-		return ((lc_host, lc_name))
+		lc_mode = self._supportedLocales[lc_id][2]
+		return ((lc_host, lc_name, lc_mode))
 
 	def search (self, db, entry, on_search_completed_callback, *args):
 		self.searching = True
@@ -122,14 +127,14 @@ class AmazonCoverArtSearch (object):
 		self.search_next ();
 
 	def __build_url (self, keyword):
-		(lc_host, lc_name) = self.__get_locale ()
+		(lc_host, lc_name, lc_mode) = self.__get_locale ()
 
 		url = "http://" + lc_host + "/onca/xml3?f=xml"
 		url += "&t=%s" % ASSOCIATE
 		url += "&dev-t=%s" % LICENSE_KEY
 		url += "&type=%s" % 'lite'
 		url += "&locale=%s" % lc_name
-		url += "&mode=%s" % 'music'
+		url += "&mode=%s" % lc_mode
 		url += "&%s=%s" % ('KeywordSearch', urllib.quote (keyword))
 
 		return url
@@ -242,7 +247,11 @@ class AmazonCoverArtSearch (object):
 						best_match = item
 						break
 					# If we already found a best_match, just keep checking for exact one
-					elif (best_match is None) and (product_name.find (album_check) != -1):
+					# Check the results for both an album name that contains the name
+					# we're searching for, and an album name that's a substring of the
+					# name we're searching for
+					elif (best_match is None) and (product_name.find (album_check) != -1 
+                                                                  or album_check.find (product_name) != -1):
 						best_match = item
 
 			# If we still have no definite hit, use first result where artist matches
