@@ -24,6 +24,8 @@
 
 #define __EXTENSIONS__
 
+#include "config.h"
+
 #include <errno.h>
 
 #include <string.h>
@@ -35,10 +37,9 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-value.h>
 
+#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
-#include <libsoup/soup-uri.h>
 
-#include "config.h"
 #include "eel-gconf-extensions.h"
 #include "rb-audioscrobbler.h"
 #include "rb-debug.h"
@@ -61,6 +62,8 @@
 #define SCROBBLER_URL "http://post.audioscrobbler.com/"
 #define SCROBBLER_VERSION "1.1"
 
+#define USER_AGENT	"Rhythmbox/" VERSION
+
 #define SCROBBLER_DATE_FORMAT "%Y%%2D%m%%2D%d%%20%H%%3A%M%%3A%S"
 
 #define EXTRA_URI_ENCODE_CHARS	"&+"
@@ -193,14 +196,20 @@ static gboolean	     rb_audioscrobbler_timeout_cb (RBAudioscrobbler *audioscrobb
 
 static gchar *	     mkmd5 (char *string);
 static void	     rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage *msg);
-static void	     rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
-					       char *url,
-					       char *post_data,
-						   SoupMessageCallbackFn response_handler);
+
 static void	     rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler);
-static void	     rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
 static void	     rb_audioscrobbler_submit_queue (RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
+						char *url,
+						char *post_data,
+						SoupSessionCallback response_handler);
+#if defined(HAVE_LIBSOUP_2_4)
+static void	     rb_audioscrobbler_do_handshake_cb (SoupSession *session, SoupMessage *msg, gpointer user_data);
+static void	     rb_audioscrobbler_submit_queue_cb (SoupSession *session, SoupMessage *msg, gpointer user_data);
+#else
+static void	     rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
 static void	     rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data);
+#endif
 
 static void	     rb_audioscrobbler_import_settings (RBAudioscrobbler *audioscrobbler);
 static void	     rb_audioscrobbler_preferences_sync (RBAudioscrobbler *audioscrobbler);
@@ -339,6 +348,8 @@ rb_audioscrobbler_dispose (GObject *object)
 	g_return_if_fail (RB_IS_AUDIOSCROBBLER (object));
 
 	audioscrobbler = RB_AUDIOSCROBBLER (object);
+	
+	rb_debug ("disposing audioscrobbler");
 
 	if (audioscrobbler->priv->offline_play_notify_id != 0) {
 		RhythmDB *db;
@@ -499,12 +510,17 @@ rb_audioscrobbler_is_queueable (RhythmDBEntry *entry)
 	 */
 	type = rhythmdb_entry_get_entry_type (entry);
 	if (type->category != RHYTHMDB_ENTRY_NORMAL) {
+		rb_debug ("entry %s is not queueable: category not NORMAL", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 		return FALSE;
 	}
 	if (type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
+		rb_debug ("entry %s is not queueable: is a podcast post", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 		return FALSE;
 	}
 	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR)) {
+		rb_debug ("entry %s is not queueable: has playback error (%s)",
+			  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION),
+			  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR));
 		return FALSE;
 	}
 
@@ -512,12 +528,24 @@ rb_audioscrobbler_is_queueable (RhythmDBEntry *entry)
 	artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
 	duration = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
 
-	/* The specification (v1.1) tells us to ignore entries which do not
-	 * verify those conditions
+	/* The specification (v1.1) tells us to ignore entries that do not
+	 * meet these conditions
 	 */
-	return 	((duration >= 30) &&
-		 (strcmp (artist, _("Unknown")) != 0) &&
-		 (strcmp (title, _("Unknown")) != 0));
+	if (duration < 30) {
+		rb_debug ("entry %s not queueable: shorter than 30 seconds", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+		return FALSE;
+	}
+	if (strcmp (artist, _("Unknown")) == 0) {
+		rb_debug ("entry %s not queueable: artist is %s (unknown)", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION), artist);
+		return FALSE;
+	}
+	if (strcmp (title, _("Unknown")) == 0) {
+		rb_debug ("entry %s not queueable: title is %s (unknown)", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION), title);
+		return FALSE;
+	}
+
+	rb_debug ("entry %s is queueable", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+	return TRUE;
 }
 
 static AudioscrobblerEntry *
@@ -564,8 +592,8 @@ rb_audioscrobbler_add_to_queue (RBAudioscrobbler *audioscrobbler,
 static void
 maybe_add_current_song_to_queue (RBAudioscrobbler *audioscrobbler)
 {
+	gboolean got_elapsed;
 	guint elapsed;
-	int elapsed_delta;
 	AudioscrobblerEntry *cur_entry;
 
 	cur_entry = audioscrobbler->priv->currently_playing;
@@ -573,27 +601,30 @@ maybe_add_current_song_to_queue (RBAudioscrobbler *audioscrobbler)
 		return;
 	}
 
-	rb_debug ("Adding currently playing song to queue");
-
-	rb_shell_player_get_playing_time (audioscrobbler->priv->shell_player, &elapsed, NULL);
-	elapsed_delta = elapsed - audioscrobbler->priv->current_elapsed;
-	audioscrobbler->priv->current_elapsed = elapsed;
-	
-	if ((elapsed >= cur_entry->length / 2 || elapsed >= 240) && elapsed_delta < 20) {
-		time (&cur_entry->play_time);
-		if (rb_audioscrobbler_add_to_queue (audioscrobbler, cur_entry)){
-			audioscrobbler->priv->currently_playing = NULL;
-		}
+	got_elapsed = rb_shell_player_get_playing_time (audioscrobbler->priv->shell_player,
+							&elapsed,
+							NULL);
+	if (got_elapsed) {
+		int elapsed_delta = elapsed - audioscrobbler->priv->current_elapsed;
+		audioscrobbler->priv->current_elapsed = elapsed;
 		
-		rb_audioscrobbler_preferences_sync (audioscrobbler);
-	} else if (elapsed_delta > 20) {
-		rb_debug ("Skipping detected; not submitting current song");
-		/* not sure about this - what if I skip to somewhere towards
-		 * the end, but then go back and listen to the whole song?
-		 */
-		audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
-		audioscrobbler->priv->currently_playing = NULL;
+		if ((elapsed >= cur_entry->length / 2 || elapsed >= 240) && elapsed_delta < 20) {
+			rb_debug ("Adding currently playing song to queue");
+			time (&cur_entry->play_time);
+			if (rb_audioscrobbler_add_to_queue (audioscrobbler, cur_entry)){
+				audioscrobbler->priv->currently_playing = NULL;
+			}
+			
+			rb_audioscrobbler_preferences_sync (audioscrobbler);
+		} else if (elapsed_delta > 20) {
+			rb_debug ("Skipping detected; not submitting current song");
+			/* not sure about this - what if I skip to somewhere towards
+			 * the end, but then go back and listen to the whole song?
+			 */
+			audioscrobbler_entry_free (audioscrobbler->priv->currently_playing);
+			audioscrobbler->priv->currently_playing = NULL;
 
+		}
 	}
 }
 
@@ -666,18 +697,31 @@ mkmd5 (char *string)
 static void
 rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage *msg)
 {
+	gboolean successful;
 	rb_debug ("Parsing response, status=%d", msg->status_code);
-
-	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL) {
-		gchar *body;
+	
+	successful = FALSE;
+#if defined(HAVE_LIBSOUP_2_4)
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && msg->response_body->length != 0)
+		successful = TRUE;
+#else
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL)
+		successful = TRUE;
+#endif
+	if (successful) {
 		gchar **breaks;
+		int i;
+#if defined(HAVE_LIBSOUP_2_2)
+		gchar *body;
 
 		body = g_malloc0 ((msg->response).length + 1);
 		memcpy (body, (msg->response).body, (msg->response).length);
 
 		g_strstrip (body);
 		breaks = g_strsplit (body, "\n", 4);
-		int i;
+#else
+		breaks = g_strsplit (msg->response_body->data, "\n", 4);
+#endif
 
 		g_free (audioscrobbler->priv->status_msg);
 		audioscrobbler->priv->status = STATUS_OK;
@@ -749,10 +793,12 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 			audioscrobbler->priv->submit_next = time(NULL) + audioscrobbler->priv->submit_interval;
 
 		g_strfreev (breaks);
+#if defined(HAVE_LIBSOUP_2_2)
 		g_free (body);
+#endif
 	} else {
 		audioscrobbler->priv->status = REQUEST_FAILED;
-		audioscrobbler->priv->status_msg = g_strdup (soup_status_get_phrase (msg->status_code));
+		audioscrobbler->priv->status_msg = g_strdup (msg->reason_phrase);
 	}
 }
 
@@ -771,24 +817,25 @@ static void
 rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 			   char *url,
 			   char *post_data,
-			   SoupMessageCallbackFn response_handler)
+			   SoupSessionCallback response_handler)
 {
 	SoupMessage *msg;
 
 	msg = soup_message_new (post_data == NULL ? "GET" : "POST", url);
+	soup_message_headers_append (msg->request_headers, "User-Agent", USER_AGENT);
 
 	if (post_data != NULL) {
 		rb_debug ("Submitting to Audioscrobbler: %s", post_data);
 		soup_message_set_request (msg,
 					  "application/x-www-form-urlencoded",
-					  SOUP_BUFFER_SYSTEM_OWNED,
+					  SOUP_MEMORY_TAKE,
 					  post_data,
 					  strlen (post_data));
 	}
 
 	/* create soup session, if we haven't got one yet */
 	if (!audioscrobbler->priv->soup_session) {
-		SoupUri *uri;
+		SoupURI *uri;
 
 		uri = rb_proxy_config_get_libsoup_uri (audioscrobbler->priv->proxy_config);
 		audioscrobbler->priv->soup_session = soup_session_async_new_with_options (
@@ -814,7 +861,6 @@ rb_audioscrobbler_should_handshake (RBAudioscrobbler *audioscrobbler)
 	 *   - we have a non-empty username
 	 */
 	if (audioscrobbler->priv->handshake) {
-		rb_debug ("Not doing handshake; we already have one");
 		return FALSE;
 	}
 
@@ -870,8 +916,13 @@ rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler)
 }
 
 
+#if defined(HAVE_LIBSOUP_2_4)
+static void
+rb_audioscrobbler_do_handshake_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
+#else
 static void
 rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data)
+#endif
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER(user_data);
 
@@ -1028,8 +1079,13 @@ rb_g_queue_concat (GQueue *q1, GQueue *q2)
 	}
 }
 
+#if defined(HAVE_LIBSOUP_2_4)
+static void
+rb_audioscrobbler_submit_queue_cb (SoupSession *session, SoupMessage *msg, gpointer user_data)
+#else
 static void
 rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data)
+#endif
 {
 	RBAudioscrobbler *audioscrobbler = RB_AUDIOSCROBBLER (user_data);
 
@@ -1211,7 +1267,7 @@ static void
 rb_audioscrobbler_proxy_config_changed_cb (RBProxyConfig *config,
 					   RBAudioscrobbler *audioscrobbler)
 {
-	SoupUri *uri;
+	SoupURI *uri;
 
 	if (audioscrobbler->priv->soup_session) {
 		uri = rb_proxy_config_get_libsoup_uri (config);
@@ -1274,6 +1330,7 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 				   RhythmDBEntry *entry,
 				   RBAudioscrobbler *audioscrobbler)
 {
+	gboolean got_time;
 	guint time;
 
 	if (audioscrobbler->priv->currently_playing != NULL) {
@@ -1282,14 +1339,22 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 	}
 
 	if (entry == NULL) {
+		rb_debug ("called with no playing entry");
 		return;
 	}
+	rb_debug ("new entry: %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
-	rb_shell_player_get_playing_time (audioscrobbler->priv->shell_player,
-					  &time, NULL);
-	audioscrobbler->priv->current_elapsed = (int) time;
+	got_time = rb_shell_player_get_playing_time (audioscrobbler->priv->shell_player,
+						     &time,
+						     NULL);
+	if (got_time) {
+		audioscrobbler->priv->current_elapsed = (int) time;
+	} else {
+		rb_debug ("didn't get playing time; assuming 0");
+		audioscrobbler->priv->current_elapsed = 0;
+	}
 
-	if (rb_audioscrobbler_is_queueable (entry) && (time < 15)) {
+	if (rb_audioscrobbler_is_queueable (entry) && (got_time == FALSE || time < 15)) {
 		AudioscrobblerEntry *as_entry;
 		
 		/* even if it's the same song, it's being played again from
