@@ -42,8 +42,8 @@
 
 #include <gconf/gconf-value.h>
 
+#include "rb-soup-compat.h"
 #include <libsoup/soup.h>
-#include <libsoup/soup-uri.h>
 
 #include "md5.h"
 
@@ -73,6 +73,9 @@
 #define RB_LASTFM_VERSION "1.1.1"
 #define EXTRA_URI_ENCODE_CHARS	"&+"
 
+#define LAST_FM_NO_COVER_IMAGE "http://static.last.fm/depth/catalogue/noimage/cover_med.gif"
+
+#define USER_AGENT "Rhythmbox/" VERSION
 
 static void rb_lastfm_source_class_init (RBLastfmSourceClass *klass);
 static void rb_lastfm_source_init (RBLastfmSource *source);
@@ -98,8 +101,12 @@ static char* rb_lastfm_source_get_playback_uri (RhythmDBEntry *entry, gpointer d
 static void rb_lastfm_perform (RBLastfmSource *lastfm,
 			       const char *url,
 			       char *post_data, /* this takes ownership */
-			       SoupMessageCallbackFn response_handler);
+			       SoupSessionCallback response_handler);
+#if defined(HAVE_LIBSOUP_2_4)
+static void rb_lastfm_message_cb (SoupSession *session, SoupMessage *req, gpointer user_data);
+#else
 static void rb_lastfm_message_cb (SoupMessage *req, gpointer user_data);
+#endif
 static void rb_lastfm_change_station (RBLastfmSource *source, const char *station);
 
 static void rb_lastfm_proxy_config_changed_cb (RBProxyConfig *config,
@@ -121,27 +128,29 @@ static void impl_get_status (RBSource *asource, char **text, char **progress_tex
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static void impl_activate (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
+static guint impl_want_uri (RBSource *source, const char *uri);
+static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
 
-static void rb_lastfm_source_new_station (char *uri, char *title, RBLastfmSource *source);
+static void rb_lastfm_source_new_station (const char *uri, const char *title, RBLastfmSource *source);
 static void rb_lastfm_source_skip_track (GtkAction *action, RBLastfmSource *source);
 static void rb_lastfm_source_love_track (GtkAction *action, RBLastfmSource *source);
 static void rb_lastfm_source_ban_track (GtkAction *action, RBLastfmSource *source);
-static char *rb_lastfm_source_title_from_uri (char *uri);
+static char *rb_lastfm_source_title_from_uri (const char *uri);
 static void rb_lastfm_source_add_station_cb (GtkButton *button, gpointer *data);
 static void rb_lastfm_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry, RBLastfmSource *source);
 
-static void rb_lastfm_source_new_song_cb (GObject *player_backend, gpointer data, RBLastfmSource *source);
+static void rb_lastfm_source_new_song_cb (GObject *player_backend,
+					  gpointer stream_data,
+					  gpointer data,
+					  RBLastfmSource *source);
 static void rb_lastfm_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry, RBLastfmSource *source);
 
 static void show_entry_popup (RBEntryView *view,
 			      gboolean over_entry,
 			      RBSource *source);
 
-#ifdef HAVE_GSTREAMER_0_10
 /* can't be bothered creating a whole header file just for this: */
 GType rb_lastfm_src_get_type (void);
-#endif
-
 
 static const char* const radio_options[][3] = {
 	{N_("Similar Artists radio"), "lastfm://artist/%s/similarartists", N_("Artists similar to %s")},
@@ -246,6 +255,8 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_activate = impl_activate;
 	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_want_uri = impl_want_uri;
+	source_class->impl_add_uri = impl_add_uri;
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
@@ -263,32 +274,17 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_type_class_add_private (klass, sizeof (RBLastfmSourcePrivate));
 
-#ifdef HAVE_GSTREAMER_0_10
 	rb_lastfm_src_get_type ();
-#endif
 }
 
 static void
 rb_lastfm_source_init (RBLastfmSource *source)
 {
-	gint size;
-	GdkPixbuf *pixbuf;
-
 	source->priv = RB_LASTFM_SOURCE_GET_PRIVATE (source);
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
-
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   "stock_channel",
-					   size,
-					   0, NULL);
-	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
-	if (pixbuf != NULL) {
-		g_object_unref (pixbuf);
-	}
 }
 
 static void
@@ -531,6 +527,7 @@ rb_lastfm_source_new (RBShell *shell)
 					  "shell", shell,
 					  "entry-type", entry_type,
 					  "proxy-config", proxy_config,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
 	rb_shell_register_entry_type_for_source (shell, source, entry_type);
 
@@ -732,30 +729,29 @@ static void
 rb_lastfm_perform (RBLastfmSource *source,
 		   const char *url,
 		   char *post_data,
-		   SoupMessageCallbackFn response_handler)
+		   SoupSessionCallback response_handler)
 {
 	SoupMessage *msg;
 	msg = soup_message_new ("GET", url);
+	soup_message_headers_append (msg->request_headers, "User-Agent", USER_AGENT);
 
 	if (msg == NULL)
 		return;
 
-	soup_message_set_http_version (msg, SOUP_HTTP_1_1);
-
 	rb_debug ("Last.fm communicating with %s", url);
 
 	if (post_data != NULL) {
 		rb_debug ("POST data: %s", post_data);
 		soup_message_set_request (msg,
 					  "application/x-www-form-urlencoded",
-					  SOUP_BUFFER_SYSTEM_OWNED,
+					  SOUP_MEMORY_TAKE,
 					  post_data,
 					  strlen (post_data));
 	}
 
 	/* create soup session, if we haven't got one yet */
 	if (!source->priv->soup_session) {
-		SoupUri *uri;
+		SoupURI *uri;
 
 		uri = rb_proxy_config_get_libsoup_uri (source->priv->proxy_config);
 		source->priv->soup_session = soup_session_async_new_with_options (
@@ -767,27 +763,45 @@ rb_lastfm_perform (RBLastfmSource *source,
 
 	soup_session_queue_message (source->priv->soup_session,
 				    msg,
-				    (SoupMessageCallbackFn) response_handler,
+				    response_handler,
 				    source);
 	source->priv->status = COMMUNICATING;
 	rb_source_notify_status_changed (RB_SOURCE(source));
 }
 
+#if defined(HAVE_LIBSOUP_2_4)
+static void
+rb_lastfm_message_cb (SoupSession *session, SoupMessage *req, gpointer user_data)
+#else
 static void
 rb_lastfm_message_cb (SoupMessage *req, gpointer user_data)
+#endif
 {
 	RBLastfmSource *source = RB_LASTFM_SOURCE (user_data);
-	char *body;
 	char **pieces;
 	int i;
+	const char *body;
+
+#if defined(HAVE_LIBSOUP_2_2)
+	char *free_body;
 
 	if ((req->response).body == NULL) {
 		rb_debug ("Lastfm: Server failed to respond");
 		return;
 	}
 
-	body = g_malloc0 ((req->response).length + 1);
-	memcpy (body, (req->response).body, (req->response).length);
+	free_body = g_malloc0 ((req->response).length + 1);
+	memcpy (free_body, (req->response).body, (req->response).length);
+	g_strstrip (free_body);
+
+	body = free_body;
+#else
+	if (req->response_body->length == 0) {
+		rb_debug ("Lastfm: Server failed to respond");
+		return;
+	}
+	body = req->response_body->data;
+#endif
 
 	rb_debug ("response body: %s", body);
 
@@ -795,8 +809,7 @@ rb_lastfm_message_cb (SoupMessage *req, gpointer user_data)
 		source->priv->status = NO_ARTIST;
 	}
 
-	g_strstrip (body);
-	pieces = g_strsplit (body, "\n", 6);
+	pieces = g_strsplit (body, "\n", 0);
 	for (i = 0; pieces[i] != NULL; i++) {
 		gchar **values = g_strsplit (pieces[i], "=", 2);
 		if (strcmp (values[0], "session") == 0) {
@@ -864,10 +877,14 @@ rb_lastfm_message_cb (SoupMessage *req, gpointer user_data)
 			rhythmdb_commit (source->priv->db);
 
 		}
+
+		g_strfreev (values);
 	}
 
 	g_strfreev (pieces);
-	g_free (body);
+#if defined(HAVE_LIBSOUP_2_2)
+	g_free (free_body);
+#endif
 
 	/* doesn't work yet
 	if (source->priv->pending_entry) {
@@ -906,7 +923,7 @@ static void
 rb_lastfm_proxy_config_changed_cb (RBProxyConfig *config,
 					   RBLastfmSource *source)
 {
-	SoupUri *uri;
+	SoupURI *uri;
 
 	if (source->priv->soup_session) {
 		uri = rb_proxy_config_get_libsoup_uri (config);
@@ -919,12 +936,12 @@ rb_lastfm_proxy_config_changed_cb (RBProxyConfig *config,
 }
 
 static void
-rb_lastfm_source_new_station (char *uri, char *title, RBLastfmSource *source)
+rb_lastfm_source_new_station (const char *uri, const char *title, RBLastfmSource *source)
 {
 	RhythmDBEntry *entry;
 	GValue v = {0,};
 
-	rb_debug ("adding lastfm: %s, %s",uri, title);
+	rb_debug ("adding lastfm: %s, %s", uri, title);
 
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
 	if (entry) {
@@ -981,21 +998,47 @@ rb_lastfm_source_command (RBLastfmSource *source, const char *query_string, cons
 }
 
 static void
-rb_lastfm_source_love_track (GtkAction *action, RBLastfmSource *source)
+rb_lastfm_source_love_track (GtkAction *run_action, RBLastfmSource *source)
 {
+	GtkAction *action;
+
 	rb_lastfm_source_command (source, "command=love", _("Marking song loved..."));
+
+	/* disable love/ban */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
-rb_lastfm_source_skip_track (GtkAction *action, RBLastfmSource *source)
+rb_lastfm_source_skip_track (GtkAction *run_action, RBLastfmSource *source)
 {
+	GtkAction *action;
+
 	rb_lastfm_source_command (source, "command=skip", _("Skipping song..."));
+
+	/* disable love/ban and skip */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmSkipSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
-rb_lastfm_source_ban_track (GtkAction *action, RBLastfmSource *source)
+rb_lastfm_source_ban_track (GtkAction *run_action, RBLastfmSource *source)
 {
+	GtkAction *action;
+
 	rb_lastfm_source_command (source, "command=ban", _("Banning song..."));
+
+	/* disable love/ban */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 
@@ -1031,7 +1074,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *selection_data)
 
 
 static char *
-rb_lastfm_source_title_from_uri (char *uri)
+rb_lastfm_source_title_from_uri (const char *uri)
 {
 	char *title = NULL;
 	char *unesc_title;
@@ -1070,8 +1113,12 @@ rb_lastfm_source_title_from_uri (char *uri)
 		title = g_strdup_printf (_("%s's %s Radio"), data[3], data[4]);
 	}
 
+	if (title == NULL && strcmp(data[2], "group") == 0) {
+		title = g_strdup_printf (_("%s Group Radio"), data[3]);
+	}
+
 	if (title == NULL) {
-		title = g_strdup (g_strstrip (uri));
+		title = g_strstrip (g_strdup (uri));
 	}
 
 	g_strfreev (data);
@@ -1141,13 +1188,20 @@ rb_lastfm_source_add_station_cb (GtkButton *button, gpointer *data)
 	g_free(title);
 }
 
+#if defined(HAVE_LIBSOUP_2_4)
+static void
+rb_lastfm_source_metadata_cb (SoupSession *session, SoupMessage *req, RBLastfmSource *source)
+#else
 static void
 rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
+#endif
 {
-	char *body;
+	const char *body;
+	char *free_body;
 	char **pieces;
 	int p;
 	RhythmDBEntry *entry;
+	gboolean found_cover;
 
 	entry = rb_shell_player_get_playing_entry (source->priv->shell_player);
 	if (entry == NULL || rhythmdb_entry_get_entry_type (entry) != source->priv->entry_type) {
@@ -1156,17 +1210,26 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 	}
 
 	rb_debug ("got response to metadata request");
-	body = g_malloc0 ((req->response).length + 1);
-	memcpy (body, (req->response).body, (req->response).length);
+#if defined(HAVE_LIBSOUP_2_4)
+	body = req->response_body->data;
+	free_body = NULL;
+#else
+	free_body = g_malloc0 ((req->response).length + 1);
+	memcpy (free_body, (req->response).body, (req->response).length);
+	g_strstrip (free_body);
+	body = free_body;
+#endif
 
-	g_strstrip (body);
 	pieces = g_strsplit (body, "\n", 0);
+	found_cover = FALSE;
 
 	for (p = 0; pieces[p] != NULL; p++) {
 		gchar **values;
 
 		values = g_strsplit (pieces[p], "=", 2);
-		if (strcmp (values[0], "station") == 0) {
+		if (values[0] == NULL) {
+			rb_debug ("empty line in response body?");
+		} else if (strcmp (values[0], "station") == 0) {
 		} else if (strcmp (values[0], "station_url") == 0) {
 		} else if (strcmp (values[0], "stationfeed") == 0) {
 		} else if (strcmp (values[0], "stationfeed_url") == 0) {
@@ -1181,6 +1244,22 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 			rb_streaming_source_set_streaming_title (RB_STREAMING_SOURCE (source), values[1]);
 		} else if (strcmp (values[0], "albumcover_small") == 0) {
 		} else if (strcmp (values[0], "albumcover_medium") == 0) {
+			GValue v = {0,};
+
+			rb_debug ("albumcover -> %s", values[1]);
+			if (strcmp (values[1], LAST_FM_NO_COVER_IMAGE) == 0) {
+				rb_debug ("ignoring last.fm's no cover image");
+			} else {
+				g_value_init (&v, G_TYPE_STRING);
+				g_value_set_string (&v, values[1]);
+				rhythmdb_emit_entry_extra_metadata_notify (source->priv->db,
+									   entry,
+									   "rb:coverArt-uri",
+									   &v);
+				g_value_unset (&v);
+				found_cover = TRUE;
+			}
+
 		} else if (strcmp (values[0], "albumcover_large") == 0) {
 		} else if (strcmp (values[0], "trackprogress") == 0) {
 		} else if (strcmp (values[0], "trackduration") == 0) {
@@ -1196,7 +1275,22 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 	}
 
 	g_strfreev (pieces);
-	g_free (body);
+#if defined(HAVE_LIBSOUP_2_2)
+	g_free (free_body);
+#endif
+
+	if (found_cover == FALSE) {
+		GValue v = {0,};
+
+		/* art display plugin treats an empty URI as meaning 'no cover art' */
+		g_value_init (&v, G_TYPE_STRING);
+		g_value_set_string (&v, "");
+		rhythmdb_emit_entry_extra_metadata_notify (source->priv->db,
+							   entry,
+							   "rb:coverArt-uri",
+							   &v);
+		g_value_unset (&v);
+	}
 
 	source->priv->status = OK;
 	rb_source_notify_status_changed (RB_SOURCE (source));
@@ -1204,18 +1298,28 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 
 static void
 rb_lastfm_source_new_song_cb (GObject *player_backend,
+			      gpointer stream_data,
 			      gpointer data,
 			      RBLastfmSource *source)
 {
+	GtkAction *action;
 	char *uri;
-	rb_debug ("got new song");
 
+	rb_debug ("got new song");
 	uri = g_strdup_printf ("http://%s%s/np.php?session=%s&debug=0",
 			       source->priv->base_url,
 			       source->priv->base_path,
 			       source->priv->session);
-	rb_lastfm_perform (source, uri, NULL, (SoupMessageCallbackFn) rb_lastfm_source_metadata_cb);
+	rb_lastfm_perform (source, uri, NULL, (SoupSessionCallback) rb_lastfm_source_metadata_cb);
 	g_free (uri);
+
+	/* re-enable actions */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmSkipSong");
+	gtk_action_set_sensitive (action, TRUE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, TRUE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, TRUE);
 }
 
 static gboolean
@@ -1283,3 +1387,26 @@ show_entry_popup (RBEntryView *view,
 	}
 }
 
+guint
+impl_want_uri (RBSource *source, const char *uri)
+{
+	if (g_str_has_prefix (uri, "lastfm://"))
+		return 100;
+
+	return 0;
+}
+
+static gboolean
+impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+{
+	char *name;
+
+	if (strstr (uri, "lastfm://") == NULL)
+		return FALSE;
+
+	name = rb_lastfm_source_title_from_uri (uri);
+
+	rb_lastfm_source_new_station (uri, name, RB_LASTFM_SOURCE (source));
+	return TRUE;
+}
+
