@@ -81,16 +81,10 @@ class MagnatuneSource(rb.BrowserSource):
 		self.__download_progress = 0.0 # progress of current download(s)
 		self.purchase_filesize = 0 # total amount of bytes to download
 
+
 	def do_set_property(self, property, value):
 		if property.name == 'plugin':
 			self.__plugin = value
-
-			# we have to wait until we get the plugin to do this
-			circle_file_name = self.__plugin.find_file("magnatune_circle_small.png")
-			width, height = gtk.icon_size_lookup(gtk.ICON_SIZE_LARGE_TOOLBAR)
-			icon = gtk.gdk.pixbuf_new_from_file_at_size(circle_file_name, width, height)
-			self.set_property("icon", icon)
-
 		else:
 			raise AttributeError, 'unknown property %s' % property.name
 
