@@ -86,7 +86,7 @@ struct RhythmDBEntry_ {
 	RBRefString *musicbrainz_artistid;
 	RBRefString *musicbrainz_albumid;
 	RBRefString *musicbrainz_albumartistid;
-	RBRefString *musicbrainz_artistsortname;
+	RBRefString *artist_sortname;
 	gulong tracknum;
 	gulong discnum;
 	gulong duration;
@@ -127,6 +127,9 @@ struct RhythmDBPrivate
 	gint read_counter;
 
 	RBMetaData *metadata;
+	gboolean metadata_blocked;
+	GMutex *metadata_lock;
+	GCond *metadata_cond;
 
 	xmlChar **column_xml_names;
 
@@ -138,6 +141,7 @@ struct RhythmDBPrivate
 	GAsyncQueue *action_queue;
 	GAsyncQueue *event_queue;
 	GAsyncQueue *restored_queue;
+	GAsyncQueue *delayed_write_queue;
 	GThreadPool *query_thread_pool;
 
 	GList *stat_list;
@@ -168,6 +172,7 @@ struct RhythmDBPrivate
 
 	GCond *saving_condition;
 	GMutex *saving_mutex;
+	guint save_count;
 
 	guint event_queue_watch_id;
 	guint commit_timeout_id;
@@ -204,6 +209,8 @@ typedef struct
 	RBRefString *uri;
 	RBRefString *real_uri; /* Target of a symlink, if any */
 	RhythmDBEntryType entry_type;
+	RhythmDBEntryType ignore_type;
+	RhythmDBEntryType error_type;
 
 	GError *error;
 	RhythmDB *db;
@@ -223,7 +230,7 @@ typedef struct
 } RhythmDBEvent;
 
 /* from rhythmdb.c */
-void queue_stat_uri (const char *uri, RhythmDB *db, RhythmDBEntryType type);
+void queue_stat_uri (const char *uri, RhythmDB *db, RhythmDBEntryType type, RhythmDBEntryType ignore_type, RhythmDBEntryType error_type);
 void rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry,
 				    gboolean visibility);
 void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
