@@ -26,10 +26,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib/gi18n.h>
+#include <gsequence.h>
 
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
-#include "gsequence.h"
 #include "rb-refstring.h"
 #include "rb-tree-dnd.h"
 
@@ -48,6 +48,7 @@ typedef struct {
 	guint refcount;
 } RhythmDBPropertyModelEntry;
 
+static void rhythmdb_property_model_dispose (GObject *object);
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -113,23 +114,28 @@ enum {
 	TARGET_GENRE,
 	TARGET_ARTISTS,
 	TARGET_LOCATION,
+	TARGET_ENTRIES,
 	TARGET_URIS,
 };
 
 static const GtkTargetEntry targets_album  [] = {
 	{ "text/x-rhythmbox-album",  0, TARGET_ALBUMS },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 static const GtkTargetEntry targets_genre  [] = {
 	{ "text/x-rhythmbox-genre",  0, TARGET_GENRE },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 static const GtkTargetEntry targets_artist [] = {
 	{ "text/x-rhythmbox-artist", 0, TARGET_ARTISTS },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 static const GtkTargetEntry targets_location [] = {
 	{ "text/x-rhythmbox-location", 0, TARGET_LOCATION },
+	{ "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 
@@ -201,6 +207,7 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 	object_class->set_property = rhythmdb_property_model_set_property;
 	object_class->get_property = rhythmdb_property_model_get_property;
 
+	object_class->dispose = rhythmdb_property_model_dispose;
 	object_class->finalize = rhythmdb_property_model_finalize;
 
 	rhythmdb_property_model_signals[PRE_ROW_DELETION] =
@@ -260,9 +267,9 @@ rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface)
 static void
 rhythmdb_property_model_drag_source_init (RbTreeDragSourceIface *iface)
 {
-	iface->row_draggable = rhythmdb_property_model_row_draggable;
-	iface->drag_data_delete = rhythmdb_property_model_drag_data_delete;
-	iface->drag_data_get = rhythmdb_property_model_drag_data_get;
+	iface->rb_row_draggable = rhythmdb_property_model_row_draggable;
+	iface->rb_drag_data_delete = rhythmdb_property_model_drag_data_delete;
+	iface->rb_drag_data_get = rhythmdb_property_model_drag_data_get;
 }
 
 static gboolean
@@ -423,12 +430,45 @@ rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 	model->priv->all->string = rb_refstring_new (_("All"));
 }
 
+static void
+_prop_model_entry_cleanup (RhythmDBPropertyModelEntry *prop, gpointer data)
+{
+	rb_refstring_unref (prop->string);
+	rb_refstring_unref (prop->sort_string);
+	g_free (prop);
+}
+
+static void
+rhythmdb_property_model_dispose (GObject *object)
+{
+	RhythmDBPropertyModel *model;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RHYTHMDB_IS_PROPERTY_MODEL (object));
+
+	model = RHYTHMDB_PROPERTY_MODEL (object);
+
+	rb_debug ("disposing property model %p", model);
+
+	g_return_if_fail (model->priv != NULL);
+
+	if (model->priv->syncing_id != 0) {
+		g_source_remove (model->priv->syncing_id);
+		model->priv->syncing_id = 0;
+	}
+
+	if (model->priv->query_model != NULL) {
+		g_object_unref (model->priv->query_model);
+		model->priv->query_model = NULL;
+	}
+
+	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->dispose (object);
+}
+
 static void
 rhythmdb_property_model_finalize (GObject *object)
 {
 	RhythmDBPropertyModel *model;
-	GSequencePtr ptr;
-	GSequencePtr end_ptr;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RHYTHMDB_IS_PROPERTY_MODEL (object));
@@ -439,25 +479,14 @@ rhythmdb_property_model_finalize (GObject *object)
 
 	g_return_if_fail (model->priv != NULL);
 
-	if (model->priv->syncing_id != 0)
-		g_source_remove (model->priv->syncing_id);
-
-	end_ptr = g_sequence_get_end_ptr (model->priv->properties);
-	for (ptr = g_sequence_get_begin_ptr (model->priv->properties); ptr != end_ptr;
-	     ptr = g_sequence_ptr_next (ptr)) {
-		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (ptr);
-		rb_refstring_unref (prop->string);
-		rb_refstring_unref (prop->sort_string);
-	}
-
 	g_hash_table_destroy (model->priv->reverse_map);
+
+	g_sequence_foreach (model->priv->properties, (GFunc)_prop_model_entry_cleanup, NULL);
 	g_sequence_free (model->priv->properties);
 
 	g_hash_table_destroy (model->priv->entries);
 
-	if (model->priv->query_model != NULL) {
-		g_object_unref (model->priv->query_model);
-	}
+	g_free (model->priv->all);
 
 	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
@@ -557,17 +586,24 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	GSequencePtr ptr;
+	GSequenceIter *ptr;
 	const char *propstr;
 
+	iter.stamp = model->priv->stamp;
 	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
 
 	model->priv->all->refcount++;
 
 	if ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr))) {
-		prop = g_sequence_ptr_get_data (ptr);
+		prop = g_sequence_get (ptr);
 		prop->refcount++;
 		rb_debug ("adding \"%s\": refcount %d", propstr, prop->refcount);
+
+		iter.user_data = ptr;
+		path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+		gtk_tree_path_free (path);
+
 		return prop;
 	}
 	rb_debug ("adding new property \"%s\"", propstr);
@@ -577,7 +613,6 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
 	prop->sort_string = rb_refstring_new (rhythmdb_entry_get_string (entry, model->priv->sort_propid));
 	prop->refcount = 1;
 
-	iter.stamp = model->priv->stamp;
 	ptr = g_sequence_insert_sorted (model->priv->properties, prop,
 					(GCompareDataFunc) rhythmdb_property_model_compare,
 					model);
@@ -610,34 +645,39 @@ static void
 rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 				     const char *propstr)
 {
-	GSequencePtr ptr;
+	GSequenceIter *ptr;
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 
 	g_assert ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr)));
 
-	model->priv->all->refcount--;
-
-	prop = g_sequence_ptr_get_data (ptr);
-	rb_debug ("deleting \"%s\": refcount: %d", propstr, prop->refcount);
-	prop->refcount--;
-	if (prop->refcount > 0)
-		return;
-
 	iter.stamp = model->priv->stamp;
 	iter.user_data = ptr;
 
+	model->priv->all->refcount--;
+
+	prop = g_sequence_get (ptr);
+	rb_debug ("deleting \"%s\": refcount: %d", propstr, prop->refcount);
+	prop->refcount--;
+	if (prop->refcount > 0) {
+		path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+		gtk_tree_path_free (path);
+		return;
+	}
+
 	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
 	g_signal_emit (G_OBJECT (model), rhythmdb_property_model_signals[PRE_ROW_DELETION], 0);
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
+
 	g_sequence_remove (ptr);
 	g_hash_table_remove (model->priv->reverse_map, propstr);
 	rb_refstring_unref (prop->string);
 	rb_refstring_unref (prop->sort_string);
+
 	g_free (prop);
-	return;
 }
 
 gboolean
@@ -645,7 +685,7 @@ rhythmdb_property_model_iter_from_string (RhythmDBPropertyModel *model,
 					  const char *name,
 					  GtkTreeIter *iter)
 {
-	GSequencePtr ptr;
+	GSequenceIter *ptr;
 
 	if (name == NULL) {
 		if (iter) {
@@ -703,7 +743,7 @@ rhythmdb_property_model_get_iter (GtkTreeModel *tree_model,
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	guint index;
-	GSequencePtr ptr;
+	GSequenceIter *ptr;
 
 	index = gtk_tree_path_get_indices (path)[0];
 
@@ -717,7 +757,7 @@ rhythmdb_property_model_get_iter (GtkTreeModel *tree_model,
 	if (index >= g_sequence_get_length (model->priv->properties))
 		return FALSE;
 
-	ptr = g_sequence_get_ptr_at_pos (model->priv->properties, index);
+	ptr = g_sequence_get_iter_at_pos (model->priv->properties, index);
 
 	iter->stamp = model->priv->stamp;
 	iter->user_data = ptr;
@@ -738,14 +778,14 @@ rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
 		return gtk_tree_path_new_first ();
 	}
 
-	if (g_sequence_ptr_is_end (iter->user_data))
+	if (g_sequence_iter_is_end (iter->user_data))
 		return NULL;
 
 	path = gtk_tree_path_new ();
 	if (iter->user_data == model->priv->all)
 		gtk_tree_path_append_index (path, 0);
 	else
-		gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data) + 1);
+		gtk_tree_path_append_index (path, g_sequence_iter_get_position (iter->user_data) + 1);
 	return path;
 }
 
@@ -779,9 +819,9 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model,
 	} else {
 		RhythmDBPropertyModelEntry *prop;
 
-		g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
+		g_return_if_fail (!g_sequence_iter_is_end (iter->user_data));
 
-		prop = g_sequence_ptr_get_data (iter->user_data);
+		prop = g_sequence_get (iter->user_data);
 		switch (column) {
 		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
@@ -810,13 +850,13 @@ rhythmdb_property_model_iter_next (GtkTreeModel  *tree_model,
 	g_return_val_if_fail (iter->stamp == model->priv->stamp, FALSE);
 
 	if (iter->user_data == model->priv->all) {
-		iter->user_data = g_sequence_get_begin_ptr (model->priv->properties);
+		iter->user_data = g_sequence_get_begin_iter (model->priv->properties);
 	} else {
-		g_return_val_if_fail (!g_sequence_ptr_is_end (iter->user_data), FALSE);
-		iter->user_data = g_sequence_ptr_next (iter->user_data);
+		g_return_val_if_fail (!g_sequence_iter_is_end (iter->user_data), FALSE);
+		iter->user_data = g_sequence_iter_next (iter->user_data);
 	}
 
-	return !g_sequence_ptr_is_end (iter->user_data);
+	return !g_sequence_iter_is_end (iter->user_data);
 }
 
 static gboolean
@@ -864,15 +904,16 @@ rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
 					gint n)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
-	GSequencePtr child;
+	GSequenceIter *child;
 
 	if (parent)
 		return FALSE;
 
 	if (n != 0) {
-		child = g_sequence_get_ptr_at_pos (model->priv->properties, n);
+		/* -1 to account for the 'all' property at position 0 */
+		child = g_sequence_get_iter_at_pos (model->priv->properties, n-1);
 
-		if (g_sequence_ptr_is_end (child))
+		if (g_sequence_iter_is_end (child))
 			return FALSE;
 		iter->user_data = child;
 	} else {
@@ -911,6 +952,7 @@ rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource,
 struct QueryModelCbStruct {
 	RhythmDB *db;
 	GString *reply;
+	gint target;
 };
 
 static gboolean
@@ -919,12 +961,18 @@ query_model_cb (GtkTreeModel *query_model,
 		GtkTreeIter *iter,
  		struct QueryModelCbStruct *data)
 {
- 	const char *uri;
  	RhythmDBEntry *entry;
 
  	gtk_tree_model_get (query_model, iter, 0, &entry, -1);
- 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
- 	g_string_append (data->reply, uri);
+	if (data->target == TARGET_ENTRIES) {
+		g_string_append_printf (data->reply,
+					"%ld",
+					rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_ENTRY_ID));
+	} else if (data->target == TARGET_URIS) {
+		const char *uri;
+		uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+		g_string_append (data->reply, uri);
+	}
  	g_string_append (data->reply, "\r\n");
 
  	rhythmdb_entry_unref (entry);
@@ -963,7 +1011,7 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 		return FALSE;
 	}
 
-	if (target == TARGET_URIS) {
+	if (target == TARGET_URIS || target == TARGET_ENTRIES) {
 		RhythmDB *db = model->priv->db;
  		RhythmDBQueryModel *query_model;
  		GString* reply = g_string_new ("");
@@ -1037,6 +1085,7 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 
 		tmp.db = db;
  		tmp.reply = reply;
+		tmp.target = target;
  		/* Too bad that we're on the main thread. Why doesn't gtk call us async?
  		 * How does file-roller manage? - it seems it refuses the drop when it isn't
 		 * done unpacking. In which case, we should tweak the drop acknowledgement,
