@@ -202,7 +202,7 @@ typedef enum
 	RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID,
 	RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID,
 	RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID,
-	RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME,
+	RHYTHMDB_PROP_ARTIST_SORTNAME,
 
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
