@@ -76,7 +76,8 @@ typedef enum
 GType			rb_playlist_manager_get_type	(void);
 
 RBPlaylistManager *	rb_playlist_manager_new		(RBShell *shell,
-							 RBSourceList *sourcelist);
+						 	 RBSourceList *sourcelist,
+							 const char *playlists_file);
 
 void			rb_playlist_manager_shutdown	(RBPlaylistManager *mgr);
 gboolean 		rb_playlist_manager_parse_file	(RBPlaylistManager *mgr,
@@ -85,7 +86,7 @@ gboolean 		rb_playlist_manager_parse_file	(RBPlaylistManager *mgr,
 
 void			rb_playlist_manager_load_playlists (RBPlaylistManager *mgr);
 
-gboolean		rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, 
+gboolean		rb_playlist_manager_save_playlists (RBPlaylistManager *mgr,
 							    gboolean force);
 
 RBSource *		rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
@@ -96,7 +97,7 @@ RBSource *		rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistMana
 
 GList *			rb_playlist_manager_get_playlists (RBPlaylistManager *manager);
 
-gboolean		rb_playlist_manager_get_playlist_names (RBPlaylistManager *manager, 
+gboolean		rb_playlist_manager_get_playlist_names (RBPlaylistManager *manager,
 								gchar ***playlists,
 								GError **error);
 gboolean		rb_playlist_manager_create_static_playlist (RBPlaylistManager *manager,
