@@ -17,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
- *  
+ *
  */
 
 #ifndef __RB_SHELL_H
@@ -41,25 +41,25 @@ enum
 	RB_SHELL_ERROR_NO_SUCH_URI,
 	RB_SHELL_ERROR_NO_SUCH_PROPERTY,
 	RB_SHELL_ERROR_IMMUTABLE_PROPERTY,
-	RB_SHELL_ERROR_INVALID_PROPERTY_TYPE
+	RB_SHELL_ERROR_INVALID_PROPERTY_TYPE,
+	RB_SHELL_ERROR_NO_SOURCE_FOR_URI
 };
 
 #define RB_SHELL_ERROR rb_shell_error_quark ()
 
 GQuark rb_shell_error_quark (void);
 
-
 typedef enum
 {
 	RB_SHELL_UI_LOCATION_SIDEBAR,
 	RB_SHELL_UI_LOCATION_MAIN_TOP,
 	RB_SHELL_UI_LOCATION_MAIN_BOTTOM,
+	RB_SHELL_UI_LOCATION_MAIN_NOTEBOOK
 } RBShellUILocation;
 
 GType rb_shell_ui_location_get_type (void);
 #define RB_TYPE_SHELL_UI_LOCATION	(rb_shell_ui_location_get_type())
 
-
 typedef struct RBShellPrivate RBShellPrivate;
 
 typedef struct
@@ -76,22 +76,22 @@ typedef struct
 	/* signals */
 	void	(*visibility_changed)	(RBShell *shell, gboolean visible);
 	void	(*create_song_info)	(RBShell *shell, RBSongInfo *song_info, gboolean multi);
+	void	(*removable_media_scan_finished) (RBShell *shell);
 } RBShellClass;
 
 GType		rb_shell_get_type	(void);
 
-RBShell *	rb_shell_new		(int argc, char **argv,
-					 gboolean no_registration,
+RBShell *	rb_shell_new		(gboolean no_registration,
 					 gboolean no_update,
 					 gboolean dry_run,
-					 char *rhythmdb);
+					 char *rhythmdb,
+					 char *playlists);
 
 gboolean        rb_shell_present        (RBShell *shell, guint32 timestamp, GError **error);
 
-RhythmDBEntryType rb_shell_guess_type_for_uri (RBShell *shell, const char *uri);
+RBSource *	rb_shell_guess_source_for_uri (RBShell *shell, const char *uri);
 
 gboolean        rb_shell_add_uri        (RBShell *shell,
-					 RhythmDBEntryType entry_type,
 					 const char *uri,
 					 const char *title,
 					 const char *genre,
@@ -136,7 +136,20 @@ void            rb_shell_hidden_notify  (RBShell *shell,
 					 guint timeout,
 					 const char *primary,
 					 GtkWidget *icon,
-					 const char *secondary);
+					 const char *secondary,
+					 gboolean requested);
+void            rb_shell_hidden_notify_markup  (RBShell *shell,
+						guint timeout,
+						const char *primary_markup,
+						GtkWidget *icon,
+						const char *secondary_markup,
+						gboolean requested);
+void		rb_shell_notify_playing_entry (RBShell *shell,
+					       RhythmDBEntry *entry,
+					       gboolean requested);
+gboolean	rb_shell_do_notify (RBShell *shell,
+				    gboolean requested,
+				    GError **error);
 
 void            rb_shell_register_entry_type_for_source (RBShell *shell,
 							 RBSource *source,
@@ -148,8 +161,9 @@ gboolean        rb_shell_get_party_mode (RBShell *shell);
 
 void rb_shell_append_source (RBShell *shell, RBSource *source, RBSource *parent);
 
-void rb_shell_add_widget (RBShell *shell, GtkWidget *widget, RBShellUILocation location);
-void rb_shell_remove_widget (RBShell *shell, GtkWidget *widget, RBShellUILocation location);
+void 		rb_shell_add_widget (RBShell *shell, GtkWidget *widget, RBShellUILocation location);
+void 		rb_shell_remove_widget (RBShell *shell, GtkWidget *widget, RBShellUILocation location);
+void		rb_shell_notebook_set_page (RBShell *shell, GtkWidget *widget);
 
 G_END_DECLS
 
