@@ -281,7 +281,7 @@ struct RBPodcastSourcePrivate
 
 static GtkActionEntry rb_podcast_source_actions [] =
 {
-	{ "MusicNewPodcast", GTK_STOCK_NEW, N_("_New Podcast Feed"), "<control>P",
+	{ "MusicNewPodcast", RB_STOCK_PODCAST_NEW, N_("_New Podcast Feed..."), "<control>P",
 	  N_("Subscribe to a new Podcast Feed"),
 	  G_CALLBACK (rb_podcast_source_cmd_new_podcast) },
 
@@ -546,7 +546,7 @@ rb_podcast_source_constructor (GType type,
 
 	source->priv->paned = gtk_vpaned_new ();
 
-	gtk_idle_add ((GtkFunction) rb_podcast_source_load_finish_cb, source);
+	g_idle_add ((GSourceFunc) rb_podcast_source_load_finish_cb, source);
 
 	source->priv->podcast_mgr = rb_podcast_manager_new (source->priv->db);
 
@@ -1054,6 +1054,7 @@ rb_podcast_source_feeds_show_popup_cb (RBPropertyView *view,
 		act_delete = gtk_action_group_get_action (source->priv->action_group, "PodcastFeedDelete");
 
 		if (lst) {
+			gtk_action_set_visible (act_update, TRUE);
 			gtk_action_set_visible (act_properties, TRUE);
 			gtk_action_set_visible (act_delete, TRUE);
 		} else {
@@ -1655,20 +1656,20 @@ rb_podcast_source_load_finish_cb  (gpointer cb_data)
 
 	rb_podcast_manager_start_sync (source->priv->podcast_mgr);
 
-	g_signal_connect_after (G_OBJECT (source->priv->podcast_mgr),
+	g_signal_connect_object (G_OBJECT (source->priv->podcast_mgr),
 	 		        "start_download",
 			  	G_CALLBACK (rb_podcast_source_start_download_cb),
-			  	source);
+			  	source, 0);
 
-	g_signal_connect_after (G_OBJECT (source->priv->podcast_mgr),
+	g_signal_connect_object (G_OBJECT (source->priv->podcast_mgr),
 			  	"finish_download",
 			  	G_CALLBACK (rb_podcast_source_finish_download_cb),
-			  	source);
+			  	source, 0);
 
-	g_signal_connect_after (G_OBJECT (source->priv->podcast_mgr),
+	g_signal_connect_object (G_OBJECT (source->priv->podcast_mgr),
 			  	"feed_updates_available",
  			  	G_CALLBACK (rb_podcast_source_feed_updates_available_cb),
-			  	source);
+			  	source, 0);
 
 	return FALSE;
 }
