@@ -80,6 +80,11 @@ static void rb_static_playlist_source_non_entry_dropped (GtkTreeModel *model,
 							 const char *uri,
 							 int position,
 							 RBStaticPlaylistSource *source);
+static void rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
+						      GtkTreePath *path,
+						      GtkTreeIter *iter,
+						      gint *order_map,
+						      RBStaticPlaylistSource *source);
 
 static void search_action_changed (GtkRadioAction *action,
 				   GtkRadioAction *current,
@@ -282,6 +287,9 @@ rb_static_playlist_source_constructor (GType type,
 	g_signal_connect_object (priv->base_model, "non-entry-dropped",
 				 G_CALLBACK (rb_static_playlist_source_non_entry_dropped),
 				 source, 0);
+	g_signal_connect_object (priv->base_model, "rows-reordered",
+				 G_CALLBACK (rb_static_playlist_source_rows_reordered),
+				 source, 0);
 
 	gtk_widget_show_all (GTK_WIDGET (source));
 
@@ -656,13 +664,14 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 	rb_playlist_source_mark_dirty (psource);
 }
 
-static void
+static gboolean
 rb_static_playlist_source_add_location_cb (const char *uri,
 					   gboolean dir,
 					   RBStaticPlaylistSource *source)
 {
 	if (!dir)
 		rb_static_playlist_source_add_location_internal (source, uri, -1);
+	return TRUE;	
 }
 
 void
@@ -748,8 +757,9 @@ rb_static_playlist_source_move_entry (RBStaticPlaylistSource *source,
 				      gint index)
 {
 	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
-	RhythmDBQueryModel *model = rb_playlist_source_get_query_model (psource);
-	rhythmdb_query_model_move_entry (model, entry, index);
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+
+	rhythmdb_query_model_move_entry (priv->base_model, entry, index);
 
 	rb_playlist_source_mark_dirty (psource);
 }
@@ -781,6 +791,17 @@ rb_static_playlist_source_row_inserted (GtkTreeModel *model,
 	rhythmdb_entry_unref (entry);
 }
 
+static void
+rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
+					  GtkTreePath *path,
+					  GtkTreeIter *iter,
+					  gint *order_map,
+					  RBStaticPlaylistSource *source)
+{
+	rb_playlist_source_mark_dirty (RB_PLAYLIST_SOURCE (source));
+}
+
+
 static GList *
 impl_get_search_actions (RBSource *source)
 {
@@ -862,3 +883,13 @@ impl_want_uri (RBSource *source, const char *uri)
 
 	return 0;
 }
+
+void
+rb_static_playlist_source_shuffle_playlist(RBStaticPlaylistSource *source) 
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+
+	rhythmdb_query_model_shuffle_entries (priv->base_model);
+
+	rb_playlist_source_mark_dirty (RB_PLAYLIST_SOURCE (source));
+}
