@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -56,12 +56,15 @@ typedef struct
 	void (*properties_selected)	(RBPropertyView *view, GList *properties);
 	void (*property_activated)	(RBPropertyView *view, const char *name);
 	void (*selection_reset)		(RBPropertyView *view);
+	void (*show_popup)		(RBPropertyView *view);
 } RBPropertyViewClass;
 
 GType		rb_property_view_get_type		(void);
 
 RBPropertyView *rb_property_view_new			(RhythmDB *db, guint propid,
 							 const char *title);
+void		rb_property_view_append_column_custom	(RBPropertyView *view,
+							GtkTreeViewColumn *column);
 
 void		rb_property_view_set_selection_mode	(RBPropertyView *view,
 							 GtkSelectionMode mode);
@@ -71,6 +74,8 @@ void		rb_property_view_reset			(RBPropertyView *view);
 void		rb_property_view_set_selection		(RBPropertyView *view,
 							 const GList *names);
 
+GList *		rb_property_view_get_selection		(RBPropertyView *view);
+
 RhythmDBPropertyModel * rb_property_view_get_model	(RBPropertyView *view);
 
 void		rb_property_view_set_model		(RBPropertyView *view,
@@ -78,6 +83,11 @@ void		rb_property_view_set_model		(RBPropertyView *view,
 
 guint		rb_property_view_get_num_properties	(RBPropertyView *view);
 
+void		rb_property_view_set_search_func	(RBPropertyView *view,
+							 GtkTreeViewSearchEqualFunc func,
+							 gpointer func_data,
+							 GtkDestroyNotify notify);
+
 G_END_DECLS
 
 #endif /* __RB_PROPERTY_VIEW_H */
